# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Copyright 2023 VMware, Inc.  All rights reserved.

# AUTO GENERATED FILE -- DO NOT MODIFY!
#
# vAPI stub file for package com.vmware.vcenter.vcha.cluster.
#---------------------------------------------------------------------------

"""
The ``com.vmware.vcenter.vcha.cluster_client`` module provides classes for
redeploying and monitoring a vCenter High Availability (VCHA) Cluster after a
successful initial deployment.

"""

__author__ = 'VMware, Inc.'
__docformat__ = 'restructuredtext en'

import sys
from warnings import warn

from com.vmware.cis_client import Tasks
from vmware.vapi.stdlib.client.task import Task
from vmware.vapi.bindings import type
from vmware.vapi.bindings.converter import TypeConverter
from vmware.vapi.bindings.enum import Enum
from vmware.vapi.bindings.error import VapiError
from vmware.vapi.bindings.struct import VapiStruct
from vmware.vapi.bindings.stub import (
    ApiInterfaceStub, StubFactoryBase, VapiInterface)
from vmware.vapi.bindings.common import raise_core_exception
from vmware.vapi.data.validator import (UnionValidator, HasFieldsOfValidator)
from vmware.vapi.exception import CoreException
from vmware.vapi.lib.constants import TaskType
from vmware.vapi.lib.rest import OperationRestMetadata


class Active(VapiInterface):
    """
    The ``Active`` class provides methods to get information related to the
    active vCenter High Availability (VCHA) node. This class was added in
    vSphere API 6.7.1.
    """

    _VAPI_SERVICE_ID = 'com.vmware.vcenter.vcha.cluster.active'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _ActiveStub)
        self._VAPI_OPERATION_IDS = {}

    class Info(VapiStruct):
        """
        The ``Active.Info`` class contains the network and placement information of
        the active node of a VCHA Cluster. This class was added in vSphere API
        6.7.1.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     management=None,
                     ha=None,
                     placement=None,
                    ):
            """
            :type  management: :class:`com.vmware.vcenter.vcha_client.IpSpec`
            :param management: IP specification for the Management network. This attribute was
                added in vSphere API 6.7.1.
            :type  ha: :class:`com.vmware.vcenter.vcha_client.IpSpec` or ``None``
            :param ha: IP specification for the HA network. This attribute was added in
                vSphere API 6.7.1.
                If None, then the second NIC of the Active Node of the VCHA cluster
                is not configured.
            :type  placement: :class:`com.vmware.vcenter.vcha_client.PlacementInfo` or ``None``
            :param placement: Contains the placement information of the active node. This
                attribute was added in vSphere API 6.7.1.
                If None, the request specified that placement information of the
                active node should not be included.
            """
            self.management = management
            self.ha = ha
            self.placement = placement
            VapiStruct.__init__(self)


    Info._set_binding_type(type.StructType(
        'com.vmware.vcenter.vcha.cluster.active.info', {
            'management': type.ReferenceType('com.vmware.vcenter.vcha_client', 'IpSpec'),
            'ha': type.OptionalType(type.ReferenceType('com.vmware.vcenter.vcha_client', 'IpSpec')),
            'placement': type.OptionalType(type.ReferenceType('com.vmware.vcenter.vcha_client', 'PlacementInfo')),
        },
        Info,
        False,
        None))



    def get(self,
            vc_spec=None,
            partial=None,
            ):
        """
        Retrieves information about the active node of a VCHA cluster. This
        method was added in vSphere API 6.7.1.

        :type  vc_spec: :class:`com.vmware.vcenter.vcha_client.CredentialsSpec` or ``None``
        :param vc_spec: Contains active node's management vCenter server credentials.
            If None, then the active vCenter Server instance is assumed to be
            either self-managed or else in enhanced linked mode and managed by
            a linked vCenter Server instance.
        :type  partial: :class:`bool` or ``None``
        :param partial: If true, then return only the information that does not require
            connecting to the Active vCenter Server. 
            If false or unset, then return all the information.
            If None, then return all the information.
        :rtype: :class:`Active.Info`
        :return: Info Information about the VCHA network and placement of the active
            node.
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidArgument` 
            If the credentials provided for authentincating with the active
            node's management vCenter server are invalid.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            If the user has insufficient privilege to perform the operation. 
            
            * If ``partial`` is false or unset, then the operation execution
              requires the Global.VCServer privilege.
            * If ``partial`` is true, then the operation execution requires the
              System.Read privilege.
        :raise: :class:`com.vmware.vapi.std.errors_client.UnverifiedPeer` 
            If the SSL certificate of the management vCenter server cannot be
            validated. 
            The value of the data attribute of
            :class:`com.vmware.vapi.std.errors_client.Error` will be a class
            that contains all the attributes defined in
            :class:`com.vmware.vcenter.vcha_client.CertificateInfo`.
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidElementConfiguration` 
            If the active node is on more than one datastore.
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            If the active virtual machine is not managed by the specified
            vCenter server for the active node.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            If the management interface IP address assignment is not static.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            If any other error occurs.
        """
        return self._invoke('get',
                            {
                            'vc_spec': vc_spec,
                            'partial': partial,
                            })
class DeploymentType(VapiInterface):
    """
    The DeploymentType class provides methods to get the deployment type of a
    vCenter High Availability Cluster (VCHA Cluster). This class was added in
    vSphere API 6.7.1.
    """

    _VAPI_SERVICE_ID = 'com.vmware.vcenter.vcha.cluster.deployment_type'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _DeploymentTypeStub)
        self._VAPI_OPERATION_IDS = {}

    class Type(Enum):
        """
        The ``DeploymentType.Type`` class defines the possible deployment types for
        a VCHA Cluster. This enumeration was added in vSphere API 6.7.1.

        .. note::
            This class represents an enumerated type in the interface language
            definition. The class contains class attributes which represent the
            values in the current version of the enumerated type. Newer versions of
            the enumerated type may contain new values. To use new values of the
            enumerated type in communication with a server that supports the newer
            version of the API, you instantiate this class. See :ref:`enumerated
            type description page <enumeration_description>`.
        """
        NONE = None
        """
        VCHA Cluster is not configured. This class attribute was added in vSphere
        API 6.7.1.

        """
        AUTO = None
        """
        VCHA Cluster was deployed automatically. This class attribute was added in
        vSphere API 6.7.1.

        """
        MANUAL = None
        """
        VCHA Cluster was deployed manually. This class attribute was added in
        vSphere API 6.7.1.

        """

        def __init__(self, string):
            """
            :type  string: :class:`str`
            :param string: String value for the :class:`Type` instance.
            """
            Enum.__init__(string)

    Type._set_values({
        'NONE': Type('NONE'),
        'AUTO': Type('AUTO'),
        'MANUAL': Type('MANUAL'),
    })
    Type._set_binding_type(type.EnumType(
        'com.vmware.vcenter.vcha.cluster.deployment_type.type',
        Type))


    class Info(VapiStruct):
        """
        The ``DeploymentType.Info`` class contains the deployment type of the VCHA
        Cluster. This class was added in vSphere API 6.7.1.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     deployment_type=None,
                    ):
            """
            :type  deployment_type: :class:`DeploymentType.Type`
            :param deployment_type: Identifies the deployment type of the VCHA cluster. This attribute
                was added in vSphere API 6.7.1.
            """
            self.deployment_type = deployment_type
            VapiStruct.__init__(self)


    Info._set_binding_type(type.StructType(
        'com.vmware.vcenter.vcha.cluster.deployment_type.info', {
            'deployment_type': type.ReferenceType(__name__, 'DeploymentType.Type'),
        },
        Info,
        False,
        None))



    def get(self):
        """
        Retrieves the deployment type of a VCHA cluster. This method was added
        in vSphere API 6.7.1.


        :rtype: :class:`DeploymentType.Info`
        :return: Info structure containing the deployment type information of the
            the VCHA cluster.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            If the user has insufficient privilege to perform the operation.
            Operation execution requires the System.Read privilege.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            If any other error occurs.
        """
        return self._invoke('get', None)
class Mode(VapiInterface):
    """
    The Mode class provides methods to manage the operating mode of a vCenter
    High Availability Cluster (VCHA Cluster). This class was added in vSphere
    API 6.7.1.
    """

    _VAPI_SERVICE_ID = 'com.vmware.vcenter.vcha.cluster.mode'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _ModeStub)
        self._VAPI_OPERATION_IDS = {}
        self._VAPI_OPERATION_IDS.update({'set_task': 'set$task'})

    class ClusterMode(Enum):
        """
        The ``Mode.ClusterMode`` class defines the possible modes for a VCHA
        Cluster. This enumeration was added in vSphere API 6.7.1.

        .. note::
            This class represents an enumerated type in the interface language
            definition. The class contains class attributes which represent the
            values in the current version of the enumerated type. Newer versions of
            the enumerated type may contain new values. To use new values of the
            enumerated type in communication with a server that supports the newer
            version of the API, you instantiate this class. See :ref:`enumerated
            type description page <enumeration_description>`.
        """
        ENABLED = None
        """
        VCHA Cluster is enabled. State replication between the Active and Passive
        node is enabled and automatic failover is allowed. This class attribute was
        added in vSphere API 6.7.1.

        """
        DISABLED = None
        """
        VCHA Cluster is disabled. State replication between the Active and Passive
        node is disabled and automatic failover is not allowed. This class
        attribute was added in vSphere API 6.7.1.

        """
        MAINTENANCE = None
        """
        VCHA Cluster is in maintenance mode. State replication between the and
        Passive node is enabled but automatic failover is not allowed. This class
        attribute was added in vSphere API 6.7.1.

        """

        def __init__(self, string):
            """
            :type  string: :class:`str`
            :param string: String value for the :class:`ClusterMode` instance.
            """
            Enum.__init__(string)

    ClusterMode._set_values({
        'ENABLED': ClusterMode('ENABLED'),
        'DISABLED': ClusterMode('DISABLED'),
        'MAINTENANCE': ClusterMode('MAINTENANCE'),
    })
    ClusterMode._set_binding_type(type.EnumType(
        'com.vmware.vcenter.vcha.cluster.mode.cluster_mode',
        ClusterMode))


    class Info(VapiStruct):
        """
        The ``Mode.Info`` class contains the mode of the VCHA Cluster. This class
        was added in vSphere API 6.7.1.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     mode=None,
                    ):
            """
            :type  mode: :class:`Mode.ClusterMode`
            :param mode: Identifies the mode of the VCHA cluster. This attribute was added
                in vSphere API 6.7.1.
            """
            self.mode = mode
            VapiStruct.__init__(self)


    Info._set_binding_type(type.StructType(
        'com.vmware.vcenter.vcha.cluster.mode.info', {
            'mode': type.ReferenceType(__name__, 'Mode.ClusterMode'),
        },
        Info,
        False,
        None))



    def get(self):
        """
        Retrieves the current mode of a VCHA cluster. This method was added in
        vSphere API 6.7.1.


        :rtype: :class:`Mode.Info`
        :return: Info structure containing the mode of the the VCHA cluster.
        :raise: :class:`com.vmware.vapi.std.errors_client.NotAllowedInCurrentState` 
            If the VCHA cluster is not configured.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            If the user has insufficient privilege to perform the operation.
            Operation execution requires the System.Read privilege.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            If any other error occurs.
        """
        return self._invoke('get', None)


    def set_task(self,
            mode,
            ):
        """
        Manipulates the mode of a VCHA Cluster. Following mode transitions are
        allowed:
        enabled -> disabled - Allowed only in healthy and degraded states.
        enabled -> maintenance - Allowed only in healthy state.
        disabled -> enabled - Allowed only in healthy state.
        maintenance -> enabled - Allowed only in healthy state with all nodes
        are running the same version.
        maintenance -> disabled - Allowed only in healthy state with all nodes
        are running the same version.
        All other transitions are not allowed. 
        
        VCHA Cluster configuration remains intact in any of the cluster modes..
        This method was added in vSphere API 6.7.1.

        :type  mode: :class:`Mode.ClusterMode`
        :param mode: Clustermode to change the VCHA cluster mode to.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            If the user has insufficient privilege to perform the operation.
            Operation execution requires the Global.VCServer privilege.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            If any other error occurs.
        """
        task_id = self._invoke('set$task',
                                {
                                'mode': mode,
                                })
        task_svc = Tasks(self._config)
        task_instance = Task(task_id, task_svc, type.VoidType())
        return task_instance
class Passive(VapiInterface):
    """
    The ``Passive`` class provides methods to validate a passive's placement
    configuration and redeploy the passive node in a vCenter High Availability
    (VCHA) cluster. This class was added in vSphere API 6.7.1.
    """

    _VAPI_SERVICE_ID = 'com.vmware.vcenter.vcha.cluster.passive'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _PassiveStub)
        self._VAPI_OPERATION_IDS = {}
        self._VAPI_OPERATION_IDS.update({'redeploy_task': 'redeploy$task'})

    class CheckSpec(VapiStruct):
        """
        The ``Passive.CheckSpec`` class contains placement information for
        validation. This class was added in vSphere API 6.7.1.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     vc_spec=None,
                     placement=None,
                    ):
            """
            :type  vc_spec: :class:`com.vmware.vcenter.vcha_client.CredentialsSpec` or ``None``
            :param vc_spec: Contains the active node's management vCenter server credentials.
                This attribute was added in vSphere API 6.7.1.
                If None, then the active vCenter Server instance is assumed to be
                either self-managed or else in enhanced linked mode and managed by
                a linked vCenter Server instance.
            :type  placement: :class:`com.vmware.vcenter.vcha_client.PlacementSpec`
            :param placement: Contains the node's placement information for validation. This
                attribute was added in vSphere API 6.7.1.
            """
            self.vc_spec = vc_spec
            self.placement = placement
            VapiStruct.__init__(self)


    CheckSpec._set_binding_type(type.StructType(
        'com.vmware.vcenter.vcha.cluster.passive.check_spec', {
            'vc_spec': type.OptionalType(type.ReferenceType('com.vmware.vcenter.vcha_client', 'CredentialsSpec')),
            'placement': type.ReferenceType('com.vmware.vcenter.vcha_client', 'PlacementSpec'),
        },
        CheckSpec,
        False,
        None))


    class CheckResult(VapiStruct):
        """
        The ``Passive.CheckResult`` class contains the warnings and errors that
        will occur during the clone operation. This class was added in vSphere API
        6.7.1.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     warnings=None,
                     errors=None,
                    ):
            """
            :type  warnings: :class:`list` of :class:`com.vmware.vapi.std_client.LocalizableMessage`
            :param warnings: A list of problems which may require attention, but which are not
                fatal. This attribute was added in vSphere API 6.7.1.
            :type  errors: :class:`list` of :class:`com.vmware.vapi.std_client.LocalizableMessage`
            :param errors: A list of problems which are fatal to the operation and the
                operation will fail. This attribute was added in vSphere API 6.7.1.
            """
            self.warnings = warnings
            self.errors = errors
            VapiStruct.__init__(self)


    CheckResult._set_binding_type(type.StructType(
        'com.vmware.vcenter.vcha.cluster.passive.check_result', {
            'warnings': type.ListType(type.ReferenceType('com.vmware.vapi.std_client', 'LocalizableMessage')),
            'errors': type.ListType(type.ReferenceType('com.vmware.vapi.std_client', 'LocalizableMessage')),
        },
        CheckResult,
        False,
        None))


    class RedeploySpec(VapiStruct):
        """
        The ``Passive.RedeploySpec`` class contains the redeploy specification.
        This class was added in vSphere API 6.7.1.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     vc_spec=None,
                     placement=None,
                     ha_ip=None,
                     failover_ip=None,
                    ):
            """
            :type  vc_spec: :class:`com.vmware.vcenter.vcha_client.CredentialsSpec` or ``None``
            :param vc_spec: Contains the active node's management vCenter server credentials.
                This attribute was added in vSphere API 6.7.1.
                If None, then the active vCenter Server instance is assumed to be
                either self-managed or else in enhanced linked mode and managed by
                a linked vCenter Server instance.
            :type  placement: :class:`com.vmware.vcenter.vcha_client.PlacementSpec`
            :param placement: Contains the node's placement information. This attribute was added
                in vSphere API 6.7.1.
            :type  ha_ip: :class:`com.vmware.vcenter.vcha_client.IpSpec` or ``None``
            :param ha_ip: Contains the VCHA HA network configuration of the node. All cluster
                communication (state replication, heartbeat, cluster messages)
                happens over this network. This attribute was added in vSphere API
                6.7.1.
                If None, then the stored network configuration for the VCHA HA
                network for the passive node will be used.
            :type  failover_ip: :class:`com.vmware.vcenter.vcha_client.IpSpec` or ``None``
            :param failover_ip: Failover IP address that this node must assume after the failover
                to serve client requests. This attribute was added in vSphere API
                6.7.1.
                If None, then the public IP address of the Active vCenter Server is
                assumed.
            """
            self.vc_spec = vc_spec
            self.placement = placement
            self.ha_ip = ha_ip
            self.failover_ip = failover_ip
            VapiStruct.__init__(self)


    RedeploySpec._set_binding_type(type.StructType(
        'com.vmware.vcenter.vcha.cluster.passive.redeploy_spec', {
            'vc_spec': type.OptionalType(type.ReferenceType('com.vmware.vcenter.vcha_client', 'CredentialsSpec')),
            'placement': type.ReferenceType('com.vmware.vcenter.vcha_client', 'PlacementSpec'),
            'ha_ip': type.OptionalType(type.ReferenceType('com.vmware.vcenter.vcha_client', 'IpSpec')),
            'failover_ip': type.OptionalType(type.ReferenceType('com.vmware.vcenter.vcha_client', 'IpSpec')),
        },
        RedeploySpec,
        False,
        None))



    def check(self,
              spec,
              ):
        """
        Validates the specified passive node's placement configuration. This
        method was added in vSphere API 6.7.1.

        :type  spec: :class:`Passive.CheckSpec`
        :param spec: Contains the passive node's placement specification.
        :rtype: :class:`Passive.CheckResult`
        :return: CheckResult structure containing errors and warnings.
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidArgument` 
            If the credentials provided for authentincating with the active
            node's management vCenter server are invalid.
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidArgument` 
            If the specified resource spec is deemed invalid for the clone
            operation.
        :raise: :class:`com.vmware.vapi.std.errors_client.UnverifiedPeer` 
            If the SSL certificate of the management vCenter server cannot be
            validated.
            The value of the data attribute of
            :class:`com.vmware.vapi.std.errors_client.Error` will be a class
            that contains all the attributes defined in
            :class:`com.vmware.vcenter.vcha_client.CertificateInfo`.
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            If the active virtual machine is not managed by the specified
            vCenter server for the active node.
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidElementConfiguration` 
            If the active node is on more than one datastore.
        :raise: :class:`com.vmware.vapi.std.errors_client.NotAllowedInCurrentState` 
            If the clone operation is not allowed in the current state of the
            system.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            If the user has insufficient privilege to perform the operation.
            Operation execution requires the Global.VCServer privilege.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            If any other error occurs.
        """
        return self._invoke('check',
                            {
                            'spec': spec,
                            })


    def redeploy_task(self,
                 spec,
                 ):
        """
        Creates the passive node in a degraded cluster with node location
        information and pre-existing VCHA cluster configuration from the active
        node. This method was added in vSphere API 6.7.1.

        :type  spec: :class:`Passive.RedeploySpec`
        :param spec: Contains the passive node's redeploy specification.
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidArgument` 
            If the credentials provided for authentincating with the active
            node's management vCenter server are invalid.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            If the user has insufficient privilege to perform the operation.
            Operation execution requires the Global.VCServer privilege.
        :raise: :class:`com.vmware.vapi.std.errors_client.UnverifiedPeer` 
            If the SSL certificate of the management vCenter server cannot be
            validated.
            The value of the data attribute of
            :class:`com.vmware.vapi.std.errors_client.Error` will be a class
            that contains all the attributes defined in
            :class:`com.vmware.vcenter.vcha_client.CertificateInfo`.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            If any other error occurs.
        """
        task_id = self._invoke('redeploy$task',
                                {
                                'spec': spec,
                                })
        task_svc = Tasks(self._config)
        task_instance = Task(task_id, task_svc, type.VoidType())
        return task_instance
class Witness(VapiInterface):
    """
    The ``Witness`` class provides methods to validate a witness's placement
    configuration and redeploy the witness node in a vCenter High Availability
    (VCHA) cluster. This class was added in vSphere API 6.7.1.
    """

    _VAPI_SERVICE_ID = 'com.vmware.vcenter.vcha.cluster.witness'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _WitnessStub)
        self._VAPI_OPERATION_IDS = {}
        self._VAPI_OPERATION_IDS.update({'redeploy_task': 'redeploy$task'})

    class CheckSpec(VapiStruct):
        """
        The ``Witness.CheckSpec`` class contains placement information for
        validation. This class was added in vSphere API 6.7.1.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     vc_spec=None,
                     placement=None,
                    ):
            """
            :type  vc_spec: :class:`com.vmware.vcenter.vcha_client.CredentialsSpec` or ``None``
            :param vc_spec: Contains the active node's management vCenter server credentials.
                This attribute was added in vSphere API 6.7.1.
                If None, then the active vCenter Server instance is assumed to be
                either self-managed or else in enhanced linked mode and managed by
                a linked vCenter Server instance.
            :type  placement: :class:`com.vmware.vcenter.vcha_client.PlacementSpec`
            :param placement: Contains the node's placement information for validation. This
                attribute was added in vSphere API 6.7.1.
            """
            self.vc_spec = vc_spec
            self.placement = placement
            VapiStruct.__init__(self)


    CheckSpec._set_binding_type(type.StructType(
        'com.vmware.vcenter.vcha.cluster.witness.check_spec', {
            'vc_spec': type.OptionalType(type.ReferenceType('com.vmware.vcenter.vcha_client', 'CredentialsSpec')),
            'placement': type.ReferenceType('com.vmware.vcenter.vcha_client', 'PlacementSpec'),
        },
        CheckSpec,
        False,
        None))


    class CheckResult(VapiStruct):
        """
        The ``Witness.CheckResult`` class contains the warnings and errors that
        will occur during the clone operation. This class was added in vSphere API
        6.7.1.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     warnings=None,
                     errors=None,
                    ):
            """
            :type  warnings: :class:`list` of :class:`com.vmware.vapi.std_client.LocalizableMessage`
            :param warnings: A list of problems which may require attention, but which are not
                fatal. This attribute was added in vSphere API 6.7.1.
            :type  errors: :class:`list` of :class:`com.vmware.vapi.std_client.LocalizableMessage`
            :param errors: A list of problems which are fatal to the operation and the
                operation will fail. This attribute was added in vSphere API 6.7.1.
            """
            self.warnings = warnings
            self.errors = errors
            VapiStruct.__init__(self)


    CheckResult._set_binding_type(type.StructType(
        'com.vmware.vcenter.vcha.cluster.witness.check_result', {
            'warnings': type.ListType(type.ReferenceType('com.vmware.vapi.std_client', 'LocalizableMessage')),
            'errors': type.ListType(type.ReferenceType('com.vmware.vapi.std_client', 'LocalizableMessage')),
        },
        CheckResult,
        False,
        None))


    class RedeploySpec(VapiStruct):
        """
        The ``Witness.RedeploySpec`` class contains the redeploy specification.
        This class was added in vSphere API 6.7.1.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     vc_spec=None,
                     placement=None,
                     ha_ip=None,
                    ):
            """
            :type  vc_spec: :class:`com.vmware.vcenter.vcha_client.CredentialsSpec` or ``None``
            :param vc_spec: Contains the active node's management vCenter server credentials.
                This attribute was added in vSphere API 6.7.1.
                If None, then the active vCenter Server instance is assumed to be
                either self-managed or else in enhanced linked mode and managed by
                a linked vCenter Server instance.
            :type  placement: :class:`com.vmware.vcenter.vcha_client.PlacementSpec`
            :param placement: Contains the node's placement information. This attribute was added
                in vSphere API 6.7.1.
            :type  ha_ip: :class:`com.vmware.vcenter.vcha_client.IpSpec` or ``None``
            :param ha_ip: Contains the VCHA HA network configuration of the node. All cluster
                communication (state replication, heartbeat, cluster messages)
                happens over this network. This attribute was added in vSphere API
                6.7.1.
                If None, then the stored network configuration for the VCHA HA
                network for the witness node will be used.
            """
            self.vc_spec = vc_spec
            self.placement = placement
            self.ha_ip = ha_ip
            VapiStruct.__init__(self)


    RedeploySpec._set_binding_type(type.StructType(
        'com.vmware.vcenter.vcha.cluster.witness.redeploy_spec', {
            'vc_spec': type.OptionalType(type.ReferenceType('com.vmware.vcenter.vcha_client', 'CredentialsSpec')),
            'placement': type.ReferenceType('com.vmware.vcenter.vcha_client', 'PlacementSpec'),
            'ha_ip': type.OptionalType(type.ReferenceType('com.vmware.vcenter.vcha_client', 'IpSpec')),
        },
        RedeploySpec,
        False,
        None))



    def check(self,
              spec,
              ):
        """
        Validates the specified witness node's placement configuration. This
        method was added in vSphere API 6.7.1.

        :type  spec: :class:`Witness.CheckSpec`
        :param spec: Contains the witness node's placement specification.
        :rtype: :class:`Witness.CheckResult`
        :return: CheckResult structure containing errors and warnings.
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidArgument` 
            If the credentials provided for authentincating with the active
            node's management vCenter server are invalid.
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidArgument` 
            If the specified resource spec is deemed invalid for the clone
            operation.
        :raise: :class:`com.vmware.vapi.std.errors_client.UnverifiedPeer` 
            If the SSL certificate of the management vCenter server cannot be
            validated.
            The value of the data attribute of
            :class:`com.vmware.vapi.std.errors_client.Error` will be a class
            that contains all the attributes defined in
            :class:`com.vmware.vcenter.vcha_client.CertificateInfo`.
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            If the active virtual machine is not managed by the specified
            vCenter server for the active node.
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidElementConfiguration` 
            If the active node is on more than one datastore.
        :raise: :class:`com.vmware.vapi.std.errors_client.NotAllowedInCurrentState` 
            If the clone operation is not allowed in the current state of the
            system.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            If the user has insufficient privilege to perform the operation.
            Operation execution requires the Global.VCServer privilege.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            If any other error occurs.
        """
        return self._invoke('check',
                            {
                            'spec': spec,
                            })


    def redeploy_task(self,
                 spec,
                 ):
        """
        Creates the witness node in a degraded cluster with node location
        information and pre-existing VCHA cluster configuration from the active
        node. This method was added in vSphere API 6.7.1.

        :type  spec: :class:`Witness.RedeploySpec`
        :param spec: Contains the witness node's redeploy specification.
        :raise: :class:`com.vmware.vapi.std.errors_client.InvalidArgument` 
            If the credentials provided for authentincating with the active
            node's management vCenter server are invalid.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            If the user has insufficient privilege to perform the operation.
            Operation execution requires the Global.VCServer privilege.
        :raise: :class:`com.vmware.vapi.std.errors_client.UnverifiedPeer` 
            If the SSL certificate of the management vCenter server cannot be
            validated.
            The value of the data attribute of
            :class:`com.vmware.vapi.std.errors_client.Error` will be a class
            that contains all the attributes defined in
            :class:`com.vmware.vcenter.vcha_client.CertificateInfo`.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            If any other error occurs.
        """
        task_id = self._invoke('redeploy$task',
                                {
                                'spec': spec,
                                })
        task_svc = Tasks(self._config)
        task_instance = Task(task_id, task_svc, type.VoidType())
        return task_instance
class _ActiveStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'vc_spec': type.OptionalType(type.ReferenceType('com.vmware.vcenter.vcha_client', 'CredentialsSpec')),
            'partial': type.OptionalType(type.BooleanType()),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.invalid_argument':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidArgument'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.unverified_peer':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'UnverifiedPeer'),
            'com.vmware.vapi.std.errors.invalid_element_configuration':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidElementConfiguration'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='POST',
            url_template='/vcenter/vcha/cluster/active',
            path_variables={
            },
             header_parameters={
                 },
            query_parameters={
            }
        )

        operations = {
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType(__name__, 'Active.Info'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'get': get_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.vcenter.vcha.cluster.active',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=True)

class _DeploymentTypeStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for get operation
        get_input_type = type.StructType('operation-input', {})
        get_error_dict = {
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/vcenter/vcha/cluster/deployment-type',
            path_variables={
            },
             header_parameters={
             },
            query_parameters={
            }
        )

        operations = {
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType(__name__, 'DeploymentType.Info'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'get': get_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.vcenter.vcha.cluster.deployment_type',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=True)

class _ModeStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for get operation
        get_input_type = type.StructType('operation-input', {})
        get_error_dict = {
            'com.vmware.vapi.std.errors.not_allowed_in_current_state':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotAllowedInCurrentState'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/vcenter/vcha/cluster/mode',
            path_variables={
            },
             header_parameters={
             },
            query_parameters={
            }
        )

        # properties for set operation
        set_input_type = type.StructType('operation-input', {
            'mode': type.ReferenceType(__name__, 'Mode.ClusterMode'),
        })
        set_error_dict = {
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),

        }
        set_input_value_validator_list = [
        ]
        set_output_validator_list = [
        ]
        set_rest_metadata = OperationRestMetadata(
            http_method='PUT',
            url_template='/vcenter/vcha/cluster/mode',
            path_variables={
            },
             header_parameters={
               },
            query_parameters={
            }
        )

        operations = {
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType(__name__, 'Mode.Info'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'set$task': {
                'input_type': set_input_type,
                'output_type': type.IdType(resource_types='com.vmware.cis.TASK'),
                'errors': set_error_dict,
                'input_value_validator_list': set_input_value_validator_list,
                'output_validator_list': [],
                'task_type': TaskType.TASK_ONLY,
            },
        }
        rest_metadata = {
            'get': get_rest_metadata,
            'set': set_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.vcenter.vcha.cluster.mode',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=True)

class _PassiveStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for check operation
        check_input_type = type.StructType('operation-input', {
            'spec': type.ReferenceType(__name__, 'Passive.CheckSpec'),
        })
        check_error_dict = {
            'com.vmware.vapi.std.errors.invalid_argument':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidArgument'),
            'com.vmware.vapi.std.errors.unverified_peer':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'UnverifiedPeer'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),
            'com.vmware.vapi.std.errors.invalid_element_configuration':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidElementConfiguration'),
            'com.vmware.vapi.std.errors.not_allowed_in_current_state':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotAllowedInCurrentState'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),

        }
        check_input_value_validator_list = [
        ]
        check_output_validator_list = [
        ]
        check_rest_metadata = OperationRestMetadata(
            http_method='POST',
            url_template='/vcenter/vcha/cluster/passive',
            path_variables={
            },
             header_parameters={
               },
            query_parameters={
            }
        )

        # properties for redeploy operation
        redeploy_input_type = type.StructType('operation-input', {
            'spec': type.ReferenceType(__name__, 'Passive.RedeploySpec'),
        })
        redeploy_error_dict = {
            'com.vmware.vapi.std.errors.invalid_argument':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidArgument'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.unverified_peer':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'UnverifiedPeer'),
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),

        }
        redeploy_input_value_validator_list = [
        ]
        redeploy_output_validator_list = [
        ]
        redeploy_rest_metadata = OperationRestMetadata(
            http_method='POST',
            url_template='/vcenter/vcha/cluster/passive',
            path_variables={
            },
             header_parameters={
               },
            query_parameters={
            }
        )

        operations = {
            'check': {
                'input_type': check_input_type,
                'output_type': type.ReferenceType(__name__, 'Passive.CheckResult'),
                'errors': check_error_dict,
                'input_value_validator_list': check_input_value_validator_list,
                'output_validator_list': check_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'redeploy$task': {
                'input_type': redeploy_input_type,
                'output_type': type.IdType(resource_types='com.vmware.cis.TASK'),
                'errors': redeploy_error_dict,
                'input_value_validator_list': redeploy_input_value_validator_list,
                'output_validator_list': [],
                'task_type': TaskType.TASK_ONLY,
            },
        }
        rest_metadata = {
            'check': check_rest_metadata,
            'redeploy': redeploy_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.vcenter.vcha.cluster.passive',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=True)

class _WitnessStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for check operation
        check_input_type = type.StructType('operation-input', {
            'spec': type.ReferenceType(__name__, 'Witness.CheckSpec'),
        })
        check_error_dict = {
            'com.vmware.vapi.std.errors.invalid_argument':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidArgument'),
            'com.vmware.vapi.std.errors.unverified_peer':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'UnverifiedPeer'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),
            'com.vmware.vapi.std.errors.invalid_element_configuration':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidElementConfiguration'),
            'com.vmware.vapi.std.errors.not_allowed_in_current_state':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotAllowedInCurrentState'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),

        }
        check_input_value_validator_list = [
        ]
        check_output_validator_list = [
        ]
        check_rest_metadata = OperationRestMetadata(
            http_method='POST',
            url_template='/vcenter/vcha/cluster/witness',
            path_variables={
            },
             header_parameters={
               },
            query_parameters={
            }
        )

        # properties for redeploy operation
        redeploy_input_type = type.StructType('operation-input', {
            'spec': type.ReferenceType(__name__, 'Witness.RedeploySpec'),
        })
        redeploy_error_dict = {
            'com.vmware.vapi.std.errors.invalid_argument':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'InvalidArgument'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.unverified_peer':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'UnverifiedPeer'),
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),

        }
        redeploy_input_value_validator_list = [
        ]
        redeploy_output_validator_list = [
        ]
        redeploy_rest_metadata = OperationRestMetadata(
            http_method='POST',
            url_template='/vcenter/vcha/cluster/witness',
            path_variables={
            },
             header_parameters={
               },
            query_parameters={
            }
        )

        operations = {
            'check': {
                'input_type': check_input_type,
                'output_type': type.ReferenceType(__name__, 'Witness.CheckResult'),
                'errors': check_error_dict,
                'input_value_validator_list': check_input_value_validator_list,
                'output_validator_list': check_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'redeploy$task': {
                'input_type': redeploy_input_type,
                'output_type': type.IdType(resource_types='com.vmware.cis.TASK'),
                'errors': redeploy_error_dict,
                'input_value_validator_list': redeploy_input_value_validator_list,
                'output_validator_list': [],
                'task_type': TaskType.TASK_ONLY,
            },
        }
        rest_metadata = {
            'check': check_rest_metadata,
            'redeploy': redeploy_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.vcenter.vcha.cluster.witness',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=True)


class StubFactory(StubFactoryBase):
    _attrs = {
        'Active': Active,
        'DeploymentType': DeploymentType,
        'Mode': Mode,
        'Passive': Passive,
        'Witness': Witness,
    }




"""
Privilege Validator interface
"""

__author__ = 'VMware, Inc.'
__copyright__ = 'Copyright 2018 VMware, Inc.  All rights reserved. -- VMware Confidential'  # pylint: disable=line-too-long

from vmware.vapi.lib.log import get_vapi_logger

# Configure logging
logger = get_vapi_logger(__name__)


class PrivilegeValidator(object):
    """
    Interface for Privilege Validation
    """

    def validate(self, user_identity, required_privileges):
        """
        Validate the privileges required for a given user identity
        """
        raise NotImplementedError

    def __hash__(self):
        return str(self).__hash__()


# Privilege Validator instance
_privilege_validator = None


def get_privilege_validator(privilege_validator=None):
    """
    Returns the singleton PrivilegeValidator instance

    :type:  :class:`str`
    :param: Privilege Validator class
    """
    global _privilege_validator
    if _privilege_validator is None:
        _privilege_validator = privilege_validator()

    return _privilege_validator



# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Copyright 2023 VMware, Inc.  All rights reserved.

# AUTO GENERATED FILE -- DO NOT MODIFY!
#
# vAPI stub file for package com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.
#---------------------------------------------------------------------------

"""
The
``com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters_client``
module provides the Trust Authority Components.

"""

__author__ = 'VMware, Inc.'
__docformat__ = 'restructuredtext en'

import sys
from warnings import warn

from com.vmware.cis_client import Tasks
from vmware.vapi.stdlib.client.task import Task
from vmware.vapi.bindings import type
from vmware.vapi.bindings.converter import TypeConverter
from vmware.vapi.bindings.enum import Enum
from vmware.vapi.bindings.error import VapiError
from vmware.vapi.bindings.struct import VapiStruct
from vmware.vapi.bindings.stub import (
    ApiInterfaceStub, StubFactoryBase, VapiInterface)
from vmware.vapi.bindings.common import raise_core_exception
from vmware.vapi.data.validator import (UnionValidator, HasFieldsOfValidator)
from vmware.vapi.exception import CoreException
from vmware.vapi.lib.constants import TaskType
from vmware.vapi.lib.rest import OperationRestMetadata


class ConsumerPrincipals(VapiInterface):
    """
    The ``ConsumerPrincipals`` class configures the token policies and STS
    trust necessary for the workload vCenter to query the trusted services for
    their status. This class was added in vSphere API 7.0.0.0.
    """

    _VAPI_SERVICE_ID = 'com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.consumer_principals'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _ConsumerPrincipalsStub)
        self._VAPI_OPERATION_IDS = {}
        self._VAPI_OPERATION_IDS.update({'create_task': 'create$task'})
        self._VAPI_OPERATION_IDS.update({'delete_task': 'delete$task'})
        self._VAPI_OPERATION_IDS.update({'get_task': 'get$task'})
        self._VAPI_OPERATION_IDS.update({'list_task': 'list$task'})

    class Health(Enum):
        """
        The ``ConsumerPrincipals.Health`` class defines the possible health states.
        This enumeration was added in vSphere API 7.0.0.0.

        .. note::
            This class represents an enumerated type in the interface language
            definition. The class contains class attributes which represent the
            values in the current version of the enumerated type. Newer versions of
            the enumerated type may contain new values. To use new values of the
            enumerated type in communication with a server that supports the newer
            version of the API, you instantiate this class. See :ref:`enumerated
            type description page <enumeration_description>`.
        """
        NONE = None
        """
        None. No status available. This class attribute was added in vSphere API
        7.0.0.0.

        """
        OK = None
        """
        OK. Health is normal. This class attribute was added in vSphere API
        7.0.0.0.

        """
        WARNING = None
        """
        Warning. Health is normal, however there is an issue that requires
        attention. This class attribute was added in vSphere API 7.0.0.0.

        """
        ERROR = None
        """
        Error. Not healthy. This class attribute was added in vSphere API 7.0.0.0.

        """

        def __init__(self, string):
            """
            :type  string: :class:`str`
            :param string: String value for the :class:`Health` instance.
            """
            Enum.__init__(string)

    Health._set_values({
        'NONE': Health('NONE'),
        'OK': Health('OK'),
        'WARNING': Health('WARNING'),
        'ERROR': Health('ERROR'),
    })
    Health._set_binding_type(type.EnumType(
        'com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.consumer_principals.health',
        Health))


    class CreateSpec(VapiStruct):
        """
        The ``ConsumerPrincipals.CreateSpec`` class contains the information
        necessary to establish trust between a workload vCenter and a Trust
        Authority Host. This class was added in vSphere API 7.0.0.0.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     certificates=None,
                     issuer_alias=None,
                     issuer=None,
                     principal=None,
                    ):
            """
            :type  certificates: :class:`list` of :class:`com.vmware.vcenter.trusted_infrastructure_client.X509CertChain`
            :param certificates: The certificates used by the vCenter STS to sign tokens. This
                attribute was added in vSphere API 7.0.0.0.
            :type  issuer_alias: :class:`str`
            :param issuer_alias: A user-friendly alias of the service which created and signed the
                security token. This attribute was added in vSphere API 7.0.0.0.
                When clients pass a value of this class as a parameter, the
                attribute must be an identifier for the resource type:
                ``com.vmware.esx.authentication.trust.security-token-issuer``. When
                methods return a value of this class as a return value, the
                attribute will be an identifier for the resource type:
                ``com.vmware.esx.authentication.trust.security-token-issuer``.
            :type  issuer: :class:`str`
            :param issuer: The service which created and signed the security token. This
                attribute was added in vSphere API 7.0.0.0.
            :type  principal: :class:`com.vmware.vcenter.trusted_infrastructure_client.StsPrincipal`
            :param principal: The principal used by the vCenter to retrieve tokens. This
                attribute was added in vSphere API 7.0.0.0.
            """
            self.certificates = certificates
            self.issuer_alias = issuer_alias
            self.issuer = issuer
            self.principal = principal
            VapiStruct.__init__(self)


    CreateSpec._set_binding_type(type.StructType(
        'com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.consumer_principals.create_spec', {
            'certificates': type.ListType(type.ReferenceType('com.vmware.vcenter.trusted_infrastructure_client', 'X509CertChain')),
            'issuer_alias': type.IdType(resource_types='com.vmware.esx.authentication.trust.security-token-issuer'),
            'issuer': type.StringType(),
            'principal': type.ReferenceType('com.vmware.vcenter.trusted_infrastructure_client', 'StsPrincipal'),
        },
        CreateSpec,
        False,
        None))


    class FilterSpec(VapiStruct):
        """
        The ``ConsumerPrincipals.FilterSpec`` class contains data which identifies
        a connection profile on the trusted vCenter. This class was added in
        vSphere API 7.0.0.0.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     id=None,
                     principals=None,
                     issuer=None,
                    ):
            """
            :type  id: :class:`set` of :class:`str` or ``None``
            :param id: The unqiue identifier of a connection profile. This attribute was
                added in vSphere API 7.0.0.0.
                When clients pass a value of this class as a parameter, the
                attribute must contain identifiers for the resource type:
                ``com.vmware.esx.authentication.clientprofile``. When methods
                return a value of this class as a return value, the attribute will
                contain identifiers for the resource type:
                ``com.vmware.esx.authentication.clientprofile``.
                If None, no filtration will be performed by ID.
            :type  principals: :class:`list` of :class:`com.vmware.vcenter.trusted_infrastructure_client.StsPrincipal` or ``None``
            :param principals: The principal used by the vCenter to retrieve tokens. This
                attribute was added in vSphere API 7.0.0.0.
                If None, no filtration will be performed by principals.
            :type  issuer: :class:`set` of :class:`str` or ``None``
            :param issuer: The service which created and signed the security token. This
                attribute was added in vSphere API 7.0.0.0.
                When clients pass a value of this class as a parameter, the
                attribute must contain identifiers for the resource type:
                ``com.vmware.esx.authentication.trust.security-token-issuer``. When
                methods return a value of this class as a return value, the
                attribute will contain identifiers for the resource type:
                ``com.vmware.esx.authentication.trust.security-token-issuer``.
                If None, no filtration will be performed by issuer.
            """
            self.id = id
            self.principals = principals
            self.issuer = issuer
            VapiStruct.__init__(self)


    FilterSpec._set_binding_type(type.StructType(
        'com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.consumer_principals.filter_spec', {
            'id': type.OptionalType(type.SetType(type.IdType())),
            'principals': type.OptionalType(type.ListType(type.ReferenceType('com.vmware.vcenter.trusted_infrastructure_client', 'StsPrincipal'))),
            'issuer': type.OptionalType(type.SetType(type.IdType())),
        },
        FilterSpec,
        False,
        None))


    class Info(VapiStruct):
        """
        The ``ConsumerPrincipals.Info`` class contains the information necessary to
        establish trust between a workload vCenter and a Trust Authority Host. This
        class was added in vSphere API 7.0.0.0.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     id=None,
                     principal=None,
                     issuer_alias=None,
                     issuer=None,
                     certificates=None,
                     health=None,
                     message=None,
                    ):
            """
            :type  id: :class:`str`
            :param id: The unqiue identifier of a connection profile. This attribute was
                added in vSphere API 7.0.0.0.
                When clients pass a value of this class as a parameter, the
                attribute must be an identifier for the resource type:
                ``com.vmware.esx.authentication.clientprofile``. When methods
                return a value of this class as a return value, the attribute will
                be an identifier for the resource type:
                ``com.vmware.esx.authentication.clientprofile``.
            :type  principal: :class:`com.vmware.vcenter.trusted_infrastructure_client.StsPrincipal`
            :param principal: The principal used by the vCenter to retrieve tokens. Currently
                this is the vCenter solution user. This attribute was added in
                vSphere API 7.0.0.0.
            :type  issuer_alias: :class:`str`
            :param issuer_alias: A user-friendly alias of the service which created and signed the
                security token. This attribute was added in vSphere API 7.0.0.0.
                When clients pass a value of this class as a parameter, the
                attribute must be an identifier for the resource type:
                ``com.vmware.esx.authentication.trust.security-token-issuer``. When
                methods return a value of this class as a return value, the
                attribute will be an identifier for the resource type:
                ``com.vmware.esx.authentication.trust.security-token-issuer``.
            :type  issuer: :class:`str`
            :param issuer: The service which created and signed the security token. This
                attribute was added in vSphere API 7.0.0.0.
            :type  certificates: :class:`list` of :class:`com.vmware.vcenter.trusted_infrastructure_client.X509CertChain`
            :param certificates: The certificates used by the vCenter STS to sign tokens. This
                attribute was added in vSphere API 7.0.0.0.
            :type  health: :class:`ConsumerPrincipals.Health`
            :param health: The consistency of the profile across the hosts in the cluster.
                This attribute was added in vSphere API 7.0.0.0.
            :type  message: :class:`com.vmware.vapi.std_client.LocalizableMessage` or ``None``
            :param message: A localizable message describing the health of the profile. This
                attribute was added in vSphere API 7.0.0.0.
                If None, the certificates won't be updated.
            """
            self.id = id
            self.principal = principal
            self.issuer_alias = issuer_alias
            self.issuer = issuer
            self.certificates = certificates
            self.health = health
            self.message = message
            VapiStruct.__init__(self)


    Info._set_binding_type(type.StructType(
        'com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.consumer_principals.info', {
            'id': type.IdType(resource_types='com.vmware.esx.authentication.clientprofile'),
            'principal': type.ReferenceType('com.vmware.vcenter.trusted_infrastructure_client', 'StsPrincipal'),
            'issuer_alias': type.IdType(resource_types='com.vmware.esx.authentication.trust.security-token-issuer'),
            'issuer': type.StringType(),
            'certificates': type.ListType(type.ReferenceType('com.vmware.vcenter.trusted_infrastructure_client', 'X509CertChain')),
            'health': type.ReferenceType(__name__, 'ConsumerPrincipals.Health'),
            'message': type.OptionalType(type.ReferenceType('com.vmware.vapi.std_client', 'LocalizableMessage')),
        },
        Info,
        False,
        None))


    class Summary(VapiStruct):
        """
        The ``ConsumerPrincipals.Summary`` class contains a summary of the
        information necessary to establish trust between a workload vCenter and a
        Trust Authority Host. This class was added in vSphere API 7.0.0.0.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     id=None,
                     principal=None,
                     issuer_alias=None,
                     issuer=None,
                    ):
            """
            :type  id: :class:`str`
            :param id: The unqiue identifier of a connection profile. This attribute was
                added in vSphere API 7.0.0.0.
                When clients pass a value of this class as a parameter, the
                attribute must be an identifier for the resource type:
                ``com.vmware.esx.authentication.clientprofile``. When methods
                return a value of this class as a return value, the attribute will
                be an identifier for the resource type:
                ``com.vmware.esx.authentication.clientprofile``.
            :type  principal: :class:`com.vmware.vcenter.trusted_infrastructure_client.StsPrincipal`
            :param principal: The principal used by the vCenter to retrieve tokens. Currently
                this is the vCenter solution user. This attribute was added in
                vSphere API 7.0.0.0.
            :type  issuer_alias: :class:`str`
            :param issuer_alias: A user-friendly alias of the service which created and signed the
                security token. This attribute was added in vSphere API 7.0.0.0.
                When clients pass a value of this class as a parameter, the
                attribute must be an identifier for the resource type:
                ``com.vmware.esx.authentication.trust.security-token-issuer``. When
                methods return a value of this class as a return value, the
                attribute will be an identifier for the resource type:
                ``com.vmware.esx.authentication.trust.security-token-issuer``.
            :type  issuer: :class:`str`
            :param issuer: The service which created and signed the security token. This
                attribute was added in vSphere API 7.0.0.0.
            """
            self.id = id
            self.principal = principal
            self.issuer_alias = issuer_alias
            self.issuer = issuer
            VapiStruct.__init__(self)


    Summary._set_binding_type(type.StructType(
        'com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.consumer_principals.summary', {
            'id': type.IdType(resource_types='com.vmware.esx.authentication.clientprofile'),
            'principal': type.ReferenceType('com.vmware.vcenter.trusted_infrastructure_client', 'StsPrincipal'),
            'issuer_alias': type.IdType(resource_types='com.vmware.esx.authentication.trust.security-token-issuer'),
            'issuer': type.StringType(),
        },
        Summary,
        False,
        None))




    def create_task(self,
               cluster,
               spec,
               ):
        """
        Creates a profile with the specified connection information on all
        hosts from a Trust Authority Cluster. This method was added in vSphere
        API 7.0.0.0.

        :type  cluster: :class:`str`
        :param cluster: The ID of the Trust Authority Cluster to configure.
            The parameter must be an identifier for the resource type:
            ``ClusterComputeResource``.
        :type  spec: :class:`ConsumerPrincipals.CreateSpec`
        :param spec: The CreateSpec specifying the connection information.
        :rtype: :class:  `vmware.vapi.stdlib.client.task.Task`
        :return: Task instance
        :raise: :class:`com.vmware.vapi.std.errors_client.AlreadyExists` 
            if a profile for the issuer already exists.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            if there is a generic error.
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            if there is no such cluster.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            if the user can not be authenticated.
        """
        task_id = self._invoke('create$task',
                                {
                                'cluster': cluster,
                                'spec': spec,
                                })
        task_svc = Tasks(self._config)
        task_instance = Task(task_id, task_svc, type.IdType(resource_types='com.vmware.esx.authentication.clientprofile'))
        return task_instance


    def delete_task(self,
               cluster,
               profile,
               ):
        """
        Removes the read-only policy configured on ESX for a specific
        principal. This method was added in vSphere API 7.0.0.0.

        :type  cluster: :class:`str`
        :param cluster: The ID of the Trust Authority Cluster to configure.
            The parameter must be an identifier for the resource type:
            ``ClusterComputeResource``.
        :type  profile: :class:`str`
        :param profile: The ID of the connection profile to modify.
            The parameter must be an identifier for the resource type:
            ``com.vmware.esx.authentication.clientprofile``.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            if there is a generic error.
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            if there is no profile configured with that ID.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            if the user can not be authenticated.
        """
        task_id = self._invoke('delete$task',
                                {
                                'cluster': cluster,
                                'profile': profile,
                                })
        task_svc = Tasks(self._config)
        task_instance = Task(task_id, task_svc, type.VoidType())
        return task_instance


    def get_task(self,
            cluster,
            profile,
            ):
        """
        Retrieve information for a specific profile. This method was added in
        vSphere API 7.0.0.0.

        :type  cluster: :class:`str`
        :param cluster: The ID of the Trust Authority Cluster on which the profile is
            configured.
            The parameter must be an identifier for the resource type:
            ``ClusterComputeResource``.
        :type  profile: :class:`str`
        :param profile: The ID of the profile.
            The parameter must be an identifier for the resource type:
            ``com.vmware.esx.authentication.clientprofile``.
        :rtype: :class:  `vmware.vapi.stdlib.client.task.Task`
        :return: Task instance
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            if there is a generic error.
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            if there is no profile configured with that ID.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            if the user can not be authenticated.
        """
        task_id = self._invoke('get$task',
                                {
                                'cluster': cluster,
                                'profile': profile,
                                })
        task_svc = Tasks(self._config)
        task_instance = Task(task_id, task_svc, type.ReferenceType(__name__, 'ConsumerPrincipals.Info'))
        return task_instance


    def list_task(self,
             cluster,
             spec=None,
             ):
        """
        Lists all policies configured on a specific cluster. This method was
        added in vSphere API 7.0.0.0.

        :type  cluster: :class:`str`
        :param cluster: The ID of the Trust Authority Cluster on which the profile is
            configured.
            The parameter must be an identifier for the resource type:
            ``ClusterComputeResource``.
        :type  spec: :class:`ConsumerPrincipals.FilterSpec` or ``None``
        :param spec: A FilterSpec specifying the profiles to be listed.
            If {\\\\@term.unset} return all policies.
        :rtype: :class:  `vmware.vapi.stdlib.client.task.Task`
        :return: Task instance
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            if there is a generic error.
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            if there is no profile configured with that ID.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            if the user can not be authenticated.
        """
        task_id = self._invoke('list$task',
                                {
                                'cluster': cluster,
                                'spec': spec,
                                })
        task_svc = Tasks(self._config)
        task_instance = Task(task_id, task_svc, type.ListType(type.ReferenceType(__name__, 'ConsumerPrincipals.Summary')))
        return task_instance
class _ConsumerPrincipalsStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for create operation
        create_input_type = type.StructType('operation-input', {
            'cluster': type.IdType(resource_types='ClusterComputeResource'),
            'spec': type.ReferenceType(__name__, 'ConsumerPrincipals.CreateSpec'),
        })
        create_error_dict = {
            'com.vmware.vapi.std.errors.already_exists':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'AlreadyExists'),
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),

        }
        create_input_value_validator_list = [
        ]
        create_output_validator_list = [
        ]
        create_rest_metadata = OperationRestMetadata(
            http_method='POST',
            url_template='/vcenter/trusted-infrastructure/trust-authority-clusters/{cluster}/consumer-principals',
            request_body_parameter='spec',
            path_variables={
                'cluster': 'cluster',
            },
            query_parameters={
            },
            dispatch_parameters={
            },
            header_parameters={
            },
            dispatch_header_parameters={
            }
        )

        # properties for delete operation
        delete_input_type = type.StructType('operation-input', {
            'cluster': type.IdType(resource_types='ClusterComputeResource'),
            'profile': type.IdType(resource_types='com.vmware.esx.authentication.clientprofile'),
        })
        delete_error_dict = {
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),

        }
        delete_input_value_validator_list = [
        ]
        delete_output_validator_list = [
        ]
        delete_rest_metadata = OperationRestMetadata(
            http_method='DELETE',
            url_template='/vcenter/trusted-infrastructure/trust-authority-clusters/{cluster}/consumer-principals/{profile}',
            path_variables={
                'cluster': 'cluster',
                'profile': 'profile',
            },
            query_parameters={
            },
            dispatch_parameters={
            },
            header_parameters={
            },
            dispatch_header_parameters={
            }
        )

        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'cluster': type.IdType(resource_types='ClusterComputeResource'),
            'profile': type.IdType(resource_types='com.vmware.esx.authentication.clientprofile'),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/vcenter/trusted-infrastructure/trust-authority-clusters/{cluster}/consumer-principals/{profile}',
            path_variables={
                'cluster': 'cluster',
                'profile': 'profile',
            },
            query_parameters={
            },
            dispatch_parameters={
            },
            header_parameters={
            },
            dispatch_header_parameters={
            }
        )

        # properties for list operation
        list_input_type = type.StructType('operation-input', {
            'cluster': type.IdType(resource_types='ClusterComputeResource'),
            'spec': type.OptionalType(type.ReferenceType(__name__, 'ConsumerPrincipals.FilterSpec')),
        })
        list_error_dict = {
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),

        }
        list_input_value_validator_list = [
        ]
        list_output_validator_list = [
        ]
        list_rest_metadata = OperationRestMetadata(
            http_method='POST',
            url_template='/vcenter/trusted-infrastructure/trust-authority-clusters/{cluster}/consumer-principals',
            request_body_parameter='spec',
            path_variables={
                'cluster': 'cluster',
            },
            query_parameters={
            },
            dispatch_parameters={
                'action': 'query',
            },
            header_parameters={
            },
            dispatch_header_parameters={
            }
        )

        operations = {
            'create$task': {
                'input_type': create_input_type,
                'output_type': type.IdType(resource_types='com.vmware.cis.TASK'),
                'errors': create_error_dict,
                'input_value_validator_list': create_input_value_validator_list,
                'output_validator_list': [],
                'task_type': TaskType.TASK_ONLY,
            },
            'delete$task': {
                'input_type': delete_input_type,
                'output_type': type.IdType(resource_types='com.vmware.cis.TASK'),
                'errors': delete_error_dict,
                'input_value_validator_list': delete_input_value_validator_list,
                'output_validator_list': [],
                'task_type': TaskType.TASK_ONLY,
            },
            'get$task': {
                'input_type': get_input_type,
                'output_type': type.IdType(resource_types='com.vmware.cis.TASK'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': [],
                'task_type': TaskType.TASK_ONLY,
            },
            'list$task': {
                'input_type': list_input_type,
                'output_type': type.IdType(resource_types='com.vmware.cis.TASK'),
                'errors': list_error_dict,
                'input_value_validator_list': list_input_value_validator_list,
                'output_validator_list': [],
                'task_type': TaskType.TASK_ONLY,
            },
        }
        rest_metadata = {
            'create': create_rest_metadata,
            'delete': delete_rest_metadata,
            'get': get_rest_metadata,
            'list': list_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.consumer_principals',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=True)


class StubFactory(StubFactoryBase):
    _attrs = {
        'ConsumerPrincipals': ConsumerPrincipals,
        'attestation': 'com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.attestation_client.StubFactory',
        'kms': 'com.vmware.vcenter.trusted_infrastructure.trust_authority_clusters.kms_client.StubFactory',
    }




# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Copyright 2023 VMware, Inc.  All rights reserved.

# AUTO GENERATED FILE -- DO NOT MODIFY!
#
# vAPI stub file for package com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.ha.
#---------------------------------------------------------------------------

"""
The
``com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.ha_client``
module provides classes to manage the vCenter Server Inventory Cluster HA
settings.

"""

__author__ = 'VMware, Inc.'
__docformat__ = 'restructuredtext en'

import sys
from warnings import warn

from vmware.vapi.bindings import type
from vmware.vapi.bindings.converter import TypeConverter
from vmware.vapi.bindings.enum import Enum
from vmware.vapi.bindings.error import VapiError
from vmware.vapi.bindings.struct import VapiStruct
from vmware.vapi.bindings.stub import (
    ApiInterfaceStub, StubFactoryBase, VapiInterface)
from vmware.vapi.bindings.common import raise_core_exception
from vmware.vapi.data.validator import (UnionValidator, HasFieldsOfValidator)
from vmware.vapi.exception import CoreException
from vmware.vapi.lib.constants import TaskType
from vmware.vapi.lib.rest import OperationRestMetadata


class AdmissionControl(VapiStruct):
    """
    The ``FailuresAndResponses`` class contains attributes describing the
    Failures and responses specific configurations of a cluster. It contains
    cluster-wide configurations for vsphere HA.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 host_failures_cluster_tolerates=None,
                 policy_type=None,
                 host_fail_over_slot_policy=None,
                 cluster_resource_percentage=None,
                 dedicated_failover_hosts=None,
                 performance_degradation_vm_tolerate=None,
                ):
        """
        :type  host_failures_cluster_tolerates: :class:`long` or ``None``
        :param host_failures_cluster_tolerates: Number of host failures that should be tolerated, still
            guaranteeing sufficient resources to restart virtual machines on
            available hosts. If not set, we assume 1.
        :type  policy_type: :class:`str` or ``None``
        :param policy_type: 
        :type  host_fail_over_slot_policy: :class:`FixedSizeSlotPolicy` or ``None``
        :param host_fail_over_slot_policy: This policy allows setting a fixed slot size
        :type  cluster_resource_percentage: :class:`ClusterResourcePercentage` or ``None``
        :param cluster_resource_percentage: 
        :type  dedicated_failover_hosts: :class:`list` of :class:`str` or ``None``
        :param dedicated_failover_hosts: List of hosts dedicated for failover.
        :type  performance_degradation_vm_tolerate: :class:`long` or ``None``
        :param performance_degradation_vm_tolerate: Percentage of resource reduction that a cluster of VMs can tolerate
            in case of a failover.
        """
        self.host_failures_cluster_tolerates = host_failures_cluster_tolerates
        self.policy_type = policy_type
        self.host_fail_over_slot_policy = host_fail_over_slot_policy
        self.cluster_resource_percentage = cluster_resource_percentage
        self.dedicated_failover_hosts = dedicated_failover_hosts
        self.performance_degradation_vm_tolerate = performance_degradation_vm_tolerate
        VapiStruct.__init__(self)


AdmissionControl._set_binding_type(type.StructType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.ha.admission_control', {
        'host_failures_cluster_tolerates': type.OptionalType(type.IntegerType()),
        'policy_type': type.OptionalType(type.StringType()),
        'host_fail_over_slot_policy': type.OptionalType(type.ReferenceType(__name__, 'FixedSizeSlotPolicy')),
        'cluster_resource_percentage': type.OptionalType(type.ReferenceType(__name__, 'ClusterResourcePercentage')),
        'dedicated_failover_hosts': type.OptionalType(type.ListType(type.StringType())),
        'performance_degradation_vm_tolerate': type.OptionalType(type.IntegerType()),
    },
    AdmissionControl,
    False,
    None))



class ClusterResourcePercentage(VapiStruct):
    """
    This class defines CPU, Memory resource percentages.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 cpu=None,
                 memory=None,
                ):
        """
        :type  cpu: :class:`long`
        :param cpu: Reserved failover CPU capacity
        :type  memory: :class:`long`
        :param memory: Reserved failover Memory capacity
        """
        self.cpu = cpu
        self.memory = memory
        VapiStruct.__init__(self)


ClusterResourcePercentage._set_binding_type(type.StructType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.ha.cluster_resource_percentage', {
        'cpu': type.IntegerType(),
        'memory': type.IntegerType(),
    },
    ClusterResourcePercentage,
    False,
    None))



class FailuresAndResponses(VapiStruct):
    """
    The ``FailuresAndResponses`` class contains attributes describing the
    Failures and responses specific configurations of a cluster. It contains
    cluster-wide configurations for vsphere HA.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 host_monitoring=None,
                 default_vm_restart_priority=None,
                 restart_priority_condition_timeout=None,
                 additional_delay=None,
                 host_isolation_response=None,
                 datastore_with_pdl_failure_response=None,
                 apd_failure_response=None,
                 response_recovery=None,
                 response_delay=None,
                 vm_monitoring=None,
                 vm_tools_monitoring_settings=None,
                ):
        """
        :type  host_monitoring: :class:`FailuresAndResponses.ServiceState` or ``None``
        :param host_monitoring: 
        :type  default_vm_restart_priority: :class:`FailuresAndResponses.RestartPriority` or ``None``
        :param default_vm_restart_priority: Restart priority for a virtual machine. 
            
            If not specified at either the cluster level or the virtual machine
            level, this will default to ``medium``. 
            If None or empty, the value is skipped.
        :type  restart_priority_condition_timeout: :class:`long` or ``None``
        :param restart_priority_condition_timeout: This setting is used to specify a maximum time the lower priority
            VMs should wait for the higher priority VMs to be ready. If the
            higher priority Vms are not ready by this time, then the lower
            priority VMs are restarted irrespective of the VM ready state. This
            timeout can be used to prevent the failover of lower priority VMs
            to be stuck infinitely. Timeout specified in seconds. To use
            cluster setting for a VM override, set to -1 in per-VM. setting. 
            If None or empty, the value is skipped.
        :type  additional_delay: :class:`long` or ``None``
        :param additional_delay: After condition has been met, a mandatory delay before starting the
            next VM restart priority.
        :type  host_isolation_response: :class:`FailuresAndResponses.IsolationResponse` or ``None``
        :param host_isolation_response: Indicates whether or not the virtual machine should be powered off
            if a host determines that it is isolated from the rest of the
            compute resource. 
            
            If not specified at either the cluster level or the virtual machine
            level, this will default to ``powerOff``. 
            If None or empty, the value is skipped.
        :type  datastore_with_pdl_failure_response: :class:`FailuresAndResponses.StorageVmReaction`
        :param datastore_with_pdl_failure_response: VM storage protection setting for storage failures categorized as
            Permenant Device Loss (PDL). PDL indicates storage device failure
            or LUN removal. In case of PDL, the failed datastore or device is
            unlikely to recover. The details of PDL are
        :type  apd_failure_response: :class:`FailuresAndResponses.StorageVmReaction`
        :param apd_failure_response: VM storage protection setting for storage failures categorized as
            All Paths Down (APD). APD is a condition where a storage has become
            inaccessible for unknown reasons. It only indicates loss of
            connectivity and does not indicate storage device failure or LUN
            removal (Permanent Device Loss or PDL)
        :type  response_recovery: :class:`FailuresAndResponses.VmReactionOnAPDCleared` or ``None``
        :param response_recovery: Action taken by VM Component Protection service for a powered on VM
            when APD condition clears after APD timeout. 
            
            This property is meaningful only when vSphere HA is turned on.
            Valid values are specified by
            :class:`FailuresAndResponses.VmReactionOnAPDCleared`. The default
            value is VmReactionOnAPDCleared#none for cluster setting and
            VmReactionOnAPDCleared#useClusterDefault for per-VM setting.
        :type  response_delay: :class:`long` or ``None``
        :param response_delay: The time interval after an APD timeout has been declared and before
            VM Component Protection service will terminate the VM. The default
            value is 180 seconds if not specified. To use cluster setting for a
            VM override, set to -1 in per-VM setting.
        :type  vm_monitoring: :class:`FailuresAndResponses.VmMonitoringState` or ``None``
        :param vm_monitoring: 
        :type  vm_tools_monitoring_settings: :class:`VmToolsMonitoringSettings` or ``None``
        :param vm_tools_monitoring_settings: 
        """
        self.host_monitoring = host_monitoring
        self.default_vm_restart_priority = default_vm_restart_priority
        self.restart_priority_condition_timeout = restart_priority_condition_timeout
        self.additional_delay = additional_delay
        self.host_isolation_response = host_isolation_response
        self.datastore_with_pdl_failure_response = datastore_with_pdl_failure_response
        self.apd_failure_response = apd_failure_response
        self.response_recovery = response_recovery
        self.response_delay = response_delay
        self.vm_monitoring = vm_monitoring
        self.vm_tools_monitoring_settings = vm_tools_monitoring_settings
        VapiStruct.__init__(self)


    class ServiceState(Enum):
        """
        Possible states of an HA service. All services support the disabled and
        enabled states.

        .. note::
            This class represents an enumerated type in the interface language
            definition. The class contains class attributes which represent the
            values in the current version of the enumerated type. Newer versions of
            the enumerated type may contain new values. To use new values of the
            enumerated type in communication with a server that supports the newer
            version of the API, you instantiate this class. See :ref:`enumerated
            type description page <enumeration_description>`.
        """
        DISABLED = None
        """
        HA service is disabled.

        """
        ENABLED = None
        """
        HA service is enabled.

        """

        def __init__(self, string):
            """
            :type  string: :class:`str`
            :param string: String value for the :class:`ServiceState` instance.
            """
            Enum.__init__(string)

    ServiceState._set_values({
        'DISABLED': ServiceState('DISABLED'),
        'ENABLED': ServiceState('ENABLED'),
    })
    ServiceState._set_binding_type(type.EnumType(
        'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.ha.failures_and_responses.service_state',
        ServiceState))

    class RestartPriority(Enum):
        """
        The enum defines virtual machine restart priority values to resolve
        resource contention. The priority determines the preference that HA gives
        to a virtual machine if sufficient capacity is not available to power on
        all failed virtual machines. For example, high priority virtual machines on
        a host get preference over low priority virtual machines.

        .. note::
            This class represents an enumerated type in the interface language
            definition. The class contains class attributes which represent the
            values in the current version of the enumerated type. Newer versions of
            the enumerated type may contain new values. To use new values of the
            enumerated type in communication with a server that supports the newer
            version of the API, you instantiate this class. See :ref:`enumerated
            type description page <enumeration_description>`.
        """
        DISABLED = None
        """
        vSphere HA is disabled for this virtual machine.

        """
        LOWEST = None
        """
        Virtual machines with this priority have the lowest chance of powering on
        after a failure if there is insufficient capacity on hosts to meet all
        virtual machine needs.

        """
        LOW = None
        """
        Virtual machines with this priority have a lower chance of powering on
        after a failure if there is insufficient capacity on hosts to meet all
        virtual machine needs.

        """
        MEDIUM = None
        """
        Virtual machines with this priority have an intermediate chance of powering
        on after a failure if there is insufficient capacity on hosts to meet all
        virtual machine needs.

        """
        HIGH = None
        """
        Virtual machines with this priority have a higher chance of powering on
        after a failure if there is insufficient capacity on hosts to meet all
        virtual machine needs.

        """
        HIGHEST = None
        """
        Virtual machines with this priority have the highest chance of powering on
        after a failure if there is insufficient capacity on hosts to meet all
        virtual machine needs.

        """
        CLUSTER_RESTART_PRIORITY = None
        """
        Virtual machines with this priority use the default restart priority
        defined for the cluster that contains this virtual machine.

        """

        def __init__(self, string):
            """
            :type  string: :class:`str`
            :param string: String value for the :class:`RestartPriority` instance.
            """
            Enum.__init__(string)

    RestartPriority._set_values({
        'DISABLED': RestartPriority('DISABLED'),
        'LOWEST': RestartPriority('LOWEST'),
        'LOW': RestartPriority('LOW'),
        'MEDIUM': RestartPriority('MEDIUM'),
        'HIGH': RestartPriority('HIGH'),
        'HIGHEST': RestartPriority('HIGHEST'),
        'CLUSTER_RESTART_PRIORITY': RestartPriority('CLUSTER_RESTART_PRIORITY'),
    })
    RestartPriority._set_binding_type(type.EnumType(
        'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.ha.failures_and_responses.restart_priority',
        RestartPriority))

    class IsolationResponse(Enum):
        """
        The enum defines values that indicate whether or not the virtual machine
        should be powered off if a host determines that it is isolated from the
        rest of the cluster.

        .. note::
            This class represents an enumerated type in the interface language
            definition. The class contains class attributes which represent the
            values in the current version of the enumerated type. Newer versions of
            the enumerated type may contain new values. To use new values of the
            enumerated type in communication with a server that supports the newer
            version of the API, you instantiate this class. See :ref:`enumerated
            type description page <enumeration_description>`.
        """
        NONE = None
        """
        Do not power off the virtual machine in the event of a host network
        isolation.

        """
        POWER_OFF = None
        """
        Power off the virtual machine in the event of a host network isolation.

        """
        SHUTDOWN = None
        """
        Shut down the virtual machine guest operating system in the event of a host
        network isolation. If the guest operating system fails to shutdown within
        five minutes, HA will initiate a forced power off. 
        
        When you use the shutdown isolation response, failover can take longer
        (compared to the response) because the virtual machine cannot fail over
        until it is shutdown.

        """
        CLUSTER_ISOLATION_RESPONSE = None
        """
        Use the default isolation response defined for the cluster that contains
        this virtual machine.

        """

        def __init__(self, string):
            """
            :type  string: :class:`str`
            :param string: String value for the :class:`IsolationResponse` instance.
            """
            Enum.__init__(string)

    IsolationResponse._set_values({
        'NONE': IsolationResponse('NONE'),
        'POWER_OFF': IsolationResponse('POWER_OFF'),
        'SHUTDOWN': IsolationResponse('SHUTDOWN'),
        'CLUSTER_ISOLATION_RESPONSE': IsolationResponse('CLUSTER_ISOLATION_RESPONSE'),
    })
    IsolationResponse._set_binding_type(type.EnumType(
        'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.ha.failures_and_responses.isolation_response',
        IsolationResponse))

    class StorageVmReaction(Enum):
        """
        The VM policy settings that determine the response to storage failures.

        .. note::
            This class represents an enumerated type in the interface language
            definition. The class contains class attributes which represent the
            values in the current version of the enumerated type. Newer versions of
            the enumerated type may contain new values. To use new values of the
            enumerated type in communication with a server that supports the newer
            version of the API, you instantiate this class. See :ref:`enumerated
            type description page <enumeration_description>`.
        """
        DISABLED = None
        """
        VM Component Protection service will not monitor or react to the component
        failure. This setting does not affect other vSphere HA services such as
        Host Monitoring or VM Health Monitoring.

        """
        WARNING = None
        """
        VM Component Protection service will monitor component failures but will
        not restart an affected VM. Rather it will notify users about the component
        failures. This setting does not affect other vSphere HA services such as
        Host Monitoring or VM Health Monitoring.

        """
        RESTART_CONSERVATIVE = None
        """
        VM Component Protection service protects VMs conservatively. With this
        setting, when the service can't determine that capacity is available to
        restart a VM, it will favor keeping the VM running.

        """
        RESTART_AGGRESSIVE = None
        """
        VM Component Protection service protects VMs aggressively. With this
        setting, the service will terminate an affected VM even if it can't
        determine that capacity exists to restart the VM.

        """
        CLUSTER_DEFAULT = None
        """
        VM will use the cluster default setting. This option is only meaningful for
        per-VM settings.

        """

        def __init__(self, string):
            """
            :type  string: :class:`str`
            :param string: String value for the :class:`StorageVmReaction` instance.
            """
            Enum.__init__(string)

    StorageVmReaction._set_values({
        'DISABLED': StorageVmReaction('DISABLED'),
        'WARNING': StorageVmReaction('WARNING'),
        'RESTART_CONSERVATIVE': StorageVmReaction('RESTART_CONSERVATIVE'),
        'RESTART_AGGRESSIVE': StorageVmReaction('RESTART_AGGRESSIVE'),
        'CLUSTER_DEFAULT': StorageVmReaction('CLUSTER_DEFAULT'),
    })
    StorageVmReaction._set_binding_type(type.EnumType(
        'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.ha.failures_and_responses.storage_vm_reaction',
        StorageVmReaction))

    class VmReactionOnAPDCleared(Enum):
        """
        If an APD condition clears after an APD timeout condition has been declared
        and before VM Component Protection service terminated the VM, the guestOS
        and application may no longer be operational.

        .. note::
            This class represents an enumerated type in the interface language
            definition. The class contains class attributes which represent the
            values in the current version of the enumerated type. Newer versions of
            the enumerated type may contain new values. To use new values of the
            enumerated type in communication with a server that supports the newer
            version of the API, you instantiate this class. See :ref:`enumerated
            type description page <enumeration_description>`.
        """
        NONE = None
        """
        VM Component Protection service will not react after APD condition is
        cleared.

        """
        RESET = None
        """
        VM Component Protection service will reset the VM after APD condition is
        cleared. Note this only applies if the subject VM is still powered on.

        """
        USE_CLUSTER_DEFAULT = None
        """
        VM will use the cluster default setting. This option is only meaningful for
        per-VM settings.

        """

        def __init__(self, string):
            """
            :type  string: :class:`str`
            :param string: String value for the :class:`VmReactionOnAPDCleared` instance.
            """
            Enum.__init__(string)

    VmReactionOnAPDCleared._set_values({
        'NONE': VmReactionOnAPDCleared('NONE'),
        'RESET': VmReactionOnAPDCleared('RESET'),
        'USE_CLUSTER_DEFAULT': VmReactionOnAPDCleared('USE_CLUSTER_DEFAULT'),
    })
    VmReactionOnAPDCleared._set_binding_type(type.EnumType(
        'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.ha.failures_and_responses.vm_reaction_on_APD_cleared',
        VmReactionOnAPDCleared))

    class VmMonitoringState(Enum):
        """
        The enum defines values that indicate the state of Virtual Machine Health
        Monitoring. Health Monitoring uses the vmTools (guest) and application
        agent heartbeat modules. You can configure HA to respond to heartbeat
        failures of either one or both modules. You can also disable the HA
        response to heartbeat failures.

        .. note::
            This class represents an enumerated type in the interface language
            definition. The class contains class attributes which represent the
            values in the current version of the enumerated type. Newer versions of
            the enumerated type may contain new values. To use new values of the
            enumerated type in communication with a server that supports the newer
            version of the API, you instantiate this class. See :ref:`enumerated
            type description page <enumeration_description>`.
        """
        VM_MONITORING_DISABLED = None
        """
        Virtual machine health monitoring is disabled. In this state, HA response
        to guest and application heartbeat failures are disabled.

        """
        VM_MONITORING_ONLY = None
        """
        HA response to guest heartbeat failure is enabled.

        """
        VM_AND_APP_MONITORING = None
        """
        HA response to both guest and application heartbeat failure is enabled.

        """

        def __init__(self, string):
            """
            :type  string: :class:`str`
            :param string: String value for the :class:`VmMonitoringState` instance.
            """
            Enum.__init__(string)

    VmMonitoringState._set_values({
        'VM_MONITORING_DISABLED': VmMonitoringState('VM_MONITORING_DISABLED'),
        'VM_MONITORING_ONLY': VmMonitoringState('VM_MONITORING_ONLY'),
        'VM_AND_APP_MONITORING': VmMonitoringState('VM_AND_APP_MONITORING'),
    })
    VmMonitoringState._set_binding_type(type.EnumType(
        'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.ha.failures_and_responses.vm_monitoring_state',
        VmMonitoringState))

FailuresAndResponses._set_binding_type(type.StructType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.ha.failures_and_responses', {
        'host_monitoring': type.OptionalType(type.ReferenceType(__name__, 'FailuresAndResponses.ServiceState')),
        'default_vm_restart_priority': type.OptionalType(type.ReferenceType(__name__, 'FailuresAndResponses.RestartPriority')),
        'restart_priority_condition_timeout': type.OptionalType(type.IntegerType()),
        'additional_delay': type.OptionalType(type.IntegerType()),
        'host_isolation_response': type.OptionalType(type.ReferenceType(__name__, 'FailuresAndResponses.IsolationResponse')),
        'datastore_with_pdl_failure_response': type.ReferenceType(__name__, 'FailuresAndResponses.StorageVmReaction'),
        'apd_failure_response': type.ReferenceType(__name__, 'FailuresAndResponses.StorageVmReaction'),
        'response_recovery': type.OptionalType(type.ReferenceType(__name__, 'FailuresAndResponses.VmReactionOnAPDCleared')),
        'response_delay': type.OptionalType(type.IntegerType()),
        'vm_monitoring': type.OptionalType(type.ReferenceType(__name__, 'FailuresAndResponses.VmMonitoringState')),
        'vm_tools_monitoring_settings': type.OptionalType(type.ReferenceType(__name__, 'VmToolsMonitoringSettings')),
    },
    FailuresAndResponses,
    False,
    None))



class FixedSizeSlotPolicy(VapiStruct):
    """
    This policy allows setting a fixed slot size

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 cpu=None,
                 memory=None,
                ):
        """
        :type  cpu: :class:`long`
        :param cpu: The cpu component of the slot size (in MHz)
        :type  memory: :class:`long`
        :param memory: The memory component of the slot size (in megabytes)
        """
        self.cpu = cpu
        self.memory = memory
        VapiStruct.__init__(self)


FixedSizeSlotPolicy._set_binding_type(type.StructType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.ha.fixed_size_slot_policy', {
        'cpu': type.IntegerType(),
        'memory': type.IntegerType(),
    },
    FixedSizeSlotPolicy,
    False,
    None))



class HaVmOverrides(VapiStruct):
    """
    The VmOverrides data object contains the HA configuration settings for vm
    as a override from cluster. All fields are optional.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 name=None,
                 vm_restart_priority=None,
                 vm_additional_delay=None,
                 vm_restart_priority_timeout=None,
                 vm_isolation_response=None,
                 vm_datastore_with_pdl_failure_response=None,
                 vm_apd_failure_response=None,
                 vm_apd_failover_delay=None,
                 vm_apd_response_recovery=None,
                 vm_monitoring=None,
                ):
        """
        :type  name: :class:`str`
        :param name: Reference to the virtual machine.
        :type  vm_restart_priority: :class:`FailuresAndResponses.RestartPriority` or ``None``
        :param vm_restart_priority: Restart priority for a virtual machine.
            If None or empty, the value is skipped.
        :type  vm_additional_delay: :class:`long` or ``None``
        :param vm_additional_delay: After condition has been met, a mandatory delay before starting the
            next VM restart priority.
        :type  vm_restart_priority_timeout: :class:`long` or ``None``
        :param vm_restart_priority_timeout: This setting is used to specify a maximum time the lower priority
            VMs should wait for the higher priority VMs to be ready. If the
            higher priority Vms are not ready by this time, then the lower
            priority VMs are restarted irrespective of the VM ready state. This
            timeout can be used to prevent the failover of lower priority VMs
            to be stuck infinitely.
            If None or empty, the value is skipped.
        :type  vm_isolation_response: :class:`FailuresAndResponses.IsolationResponse` or ``None``
        :param vm_isolation_response: Indicates whether or not the virtual machine should be powered off
            if a host determines that it is isolated from the rest of the
            compute resource. 
            
            If not specified at either the cluster level or the virtual machine
            level, this will default to ``powerOff``. 
            If None or empty, the value is skipped.
        :type  vm_datastore_with_pdl_failure_response: :class:`FailuresAndResponses.StorageVmReaction`
        :param vm_datastore_with_pdl_failure_response: VM storage protection setting for storage failures categorized as
            Permenant Device Loss (PDL). PDL indicates storage device failure
            or LUN removal. In case of PDL, the failed datastore or device is
            unlikely to recover. The details of PDL are
        :type  vm_apd_failure_response: :class:`FailuresAndResponses.StorageVmReaction`
        :param vm_apd_failure_response: VM storage protection setting for storage failures categorized as
            All Paths Down (APD). APD is a condition where a storage has become
            inaccessible for unknown reasons. It only indicates loss of
            connectivity and does not indicate storage device failure or LUN
            removal (Permanent Device Loss or PDL)
        :type  vm_apd_failover_delay: :class:`long` or ``None``
        :param vm_apd_failover_delay: The time interval after an APD timeout has been declared and before
            VM Component Protection service will terminate the VM. The default
            value is 180 seconds if not specified. To use cluster setting for a
            VM override, set to -1 in per-VM setting.
        :type  vm_apd_response_recovery: :class:`FailuresAndResponses.VmReactionOnAPDCleared` or ``None``
        :param vm_apd_response_recovery: Action taken by VM Component Protection service for a powered on VM
            when APD condition clears after APD timeout. This property is
            meaningful only when vSphere HA is turned on. Valid values are
        :type  vm_monitoring: :class:`FailuresAndResponses.VmMonitoringState` or ``None``
        :param vm_monitoring: Virtual machine health monitoring is disabled. In this state, HA
            response to guest and application heartbeat failures are disabled.
        """
        self.name = name
        self.vm_restart_priority = vm_restart_priority
        self.vm_additional_delay = vm_additional_delay
        self.vm_restart_priority_timeout = vm_restart_priority_timeout
        self.vm_isolation_response = vm_isolation_response
        self.vm_datastore_with_pdl_failure_response = vm_datastore_with_pdl_failure_response
        self.vm_apd_failure_response = vm_apd_failure_response
        self.vm_apd_failover_delay = vm_apd_failover_delay
        self.vm_apd_response_recovery = vm_apd_response_recovery
        self.vm_monitoring = vm_monitoring
        VapiStruct.__init__(self)


HaVmOverrides._set_binding_type(type.StructType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.ha.ha_vm_overrides', {
        'name': type.StringType(),
        'vm_restart_priority': type.OptionalType(type.ReferenceType(__name__, 'FailuresAndResponses.RestartPriority')),
        'vm_additional_delay': type.OptionalType(type.IntegerType()),
        'vm_restart_priority_timeout': type.OptionalType(type.IntegerType()),
        'vm_isolation_response': type.OptionalType(type.ReferenceType(__name__, 'FailuresAndResponses.IsolationResponse')),
        'vm_datastore_with_pdl_failure_response': type.ReferenceType(__name__, 'FailuresAndResponses.StorageVmReaction'),
        'vm_apd_failure_response': type.ReferenceType(__name__, 'FailuresAndResponses.StorageVmReaction'),
        'vm_apd_failover_delay': type.OptionalType(type.IntegerType()),
        'vm_apd_response_recovery': type.OptionalType(type.ReferenceType(__name__, 'FailuresAndResponses.VmReactionOnAPDCleared')),
        'vm_monitoring': type.OptionalType(type.ReferenceType(__name__, 'FailuresAndResponses.VmMonitoringState')),
    },
    HaVmOverrides,
    False,
    None))



class HeartBeatDataStores(VapiStruct):
    """
    The ``HeartBeatDataStores`` class contains attributes describing the
    HeartBeatDataStores It contains cluster-wide configurations for vsphere HA.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 heartbeat_datastore_policy=None,
                 datastores=None,
                ):
        """
        :type  heartbeat_datastore_policy: :class:`HeartBeatDataStores.HBDatastoreCandidate`
        :param heartbeat_datastore_policy: 
        :type  datastores: :class:`list` of :class:`str` or ``None``
        :param datastores: 
        """
        self.heartbeat_datastore_policy = heartbeat_datastore_policy
        self.datastores = datastores
        VapiStruct.__init__(self)


    class HBDatastoreCandidate(Enum):
        """
        The policy to determine the candidates from which vCenter Server can choose
        heartbeat datastores.

        .. note::
            This class represents an enumerated type in the interface language
            definition. The class contains class attributes which represent the
            values in the current version of the enumerated type. Newer versions of
            the enumerated type may contain new values. To use new values of the
            enumerated type in communication with a server that supports the newer
            version of the API, you instantiate this class. See :ref:`enumerated
            type description page <enumeration_description>`.
        """
        USER_SELECTED_DS = None
        """
        vCenter Server chooses heartbeat datastores from the set specified by the
        user (see #heartbeatDatastore). More specifically, datastores not included
        in the set will not be chosen. Note that if #heartbeatDatastore is empty,
        datastore heartbeating will be disabled for HA.

        """
        ALL_FEASIBLE_DS = None
        """
        vCenter Server chooses heartbeat datastores from all the feasible ones,
        i.e., the datastores that are accessible to more than one host in the
        cluster. The choice will be made without giving preference to those
        specified by the user (see #heartbeatDatastore).

        """
        ALL_FEASIBLE_DS_WITH_USER_PREFERENCE = None
        """
        vCenter Server chooses heartbeat datastores from all the feasible ones
        while giving preference to those specified by the user (see
        #heartbeatDatastore). More specifically, the datastores not included in
        #heartbeatDatastore will be chosen if and only if the specified ones are
        not sufficient.

        """

        def __init__(self, string):
            """
            :type  string: :class:`str`
            :param string: String value for the :class:`HBDatastoreCandidate` instance.
            """
            Enum.__init__(string)

    HBDatastoreCandidate._set_values({
        'USER_SELECTED_DS': HBDatastoreCandidate('USER_SELECTED_DS'),
        'ALL_FEASIBLE_DS': HBDatastoreCandidate('ALL_FEASIBLE_DS'),
        'ALL_FEASIBLE_DS_WITH_USER_PREFERENCE': HBDatastoreCandidate('ALL_FEASIBLE_DS_WITH_USER_PREFERENCE'),
    })
    HBDatastoreCandidate._set_binding_type(type.EnumType(
        'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.ha.heart_beat_data_stores.HB_datastore_candidate',
        HBDatastoreCandidate))

HeartBeatDataStores._set_binding_type(type.StructType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.ha.heart_beat_data_stores', {
        'heartbeat_datastore_policy': type.ReferenceType(__name__, 'HeartBeatDataStores.HBDatastoreCandidate'),
        'datastores': type.OptionalType(type.ListType(type.StringType())),
    },
    HeartBeatDataStores,
    False,
    None))



class VmToolsMonitoringSettings(VapiStruct):
    """
    The vim.cluster.VmToolsMonitoringSettings data object contains virtual
    machine monitoring settings that are used by the Virtual Machine Health
    Monitoring Service. The Service checks the VMware Tools heartbeat of a
    virtual machine. If heartbeats have not been received within a specified
    time interval, the Service declares the virtual machine as failed and
    resets the virtual machine. All fields are optional. In case of a
    reconfiguration, fields left unset are not changed.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 enabled=None,
                 cluster_settings=None,
                 failure_interval=None,
                 min_up_time=None,
                 max_failures=None,
                 max_failure_window=None,
                ):
        """
        :type  enabled: :class:`bool` or ``None``
        :param enabled: Flag indicating whether or not the Virtual Machine Health
            Monitoring service is enabled. 
            
            The Server does not use this property.
        :type  cluster_settings: :class:`bool` or ``None``
        :param cluster_settings: Flag indicating whether to use the cluster settings or the per VM
            settings. 
            
            The default value is true.
        :type  failure_interval: :class:`long` or ``None``
        :param failure_interval: If no heartbeat has been received for at least the specified number
            of seconds, the virtual machine is declared as failed. 
            
            The default value is 30.
        :type  min_up_time: :class:`long` or ``None``
        :param min_up_time: The number of seconds for the virtual machine's heartbeats to
            stabilize after the virtual machine has been powered on. This time
            should include the guest operating system boot-up time. The virtual
            machine monitoring will begin only after this period. 
            
            The default value is 120.
        :type  max_failures: :class:`long` or ``None``
        :param max_failures: Maximum number of failures and automated resets allowed during the
            time that :attr:`VmToolsMonitoringSettings.max_failure_window`
            specifies. If :attr:`VmToolsMonitoringSettings.max_failure_window`
            is -1 (no window), this represents the absolute number of failures
            after which automated response is stopped. 
            
            If a virtual machine exceeds this threshold, in-depth problem
            analysis is usually needed. 
            
            The default value is 3.
        :type  max_failure_window: :class:`long` or ``None``
        :param max_failure_window: The number of seconds for the window during which up to
            :attr:`VmToolsMonitoringSettings.max_failures` resets can occur
            before automated responses stop. 
            
            If set to -1, no failure window is specified. 
            
            The default value is -1.
        """
        self.enabled = enabled
        self.cluster_settings = cluster_settings
        self.failure_interval = failure_interval
        self.min_up_time = min_up_time
        self.max_failures = max_failures
        self.max_failure_window = max_failure_window
        VapiStruct.__init__(self)


VmToolsMonitoringSettings._set_binding_type(type.StructType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.ha.vm_tools_monitoring_settings', {
        'enabled': type.OptionalType(type.BooleanType()),
        'cluster_settings': type.OptionalType(type.BooleanType()),
        'failure_interval': type.OptionalType(type.IntegerType()),
        'min_up_time': type.OptionalType(type.IntegerType()),
        'max_failures': type.OptionalType(type.IntegerType()),
        'max_failure_window': type.OptionalType(type.IntegerType()),
    },
    VmToolsMonitoringSettings,
    False,
    None))



class VsphereHA(VapiStruct):
    """
    The ``DasInfo`` class contains attributes describing the HA specific
    configurations of a cluster. It contains cluster-wide configurations for
    DAS.

    .. tip::
        The arguments are used to initialize data attributes with the same
        names.
    """




    def __init__(self,
                 enabled=None,
                 failures_and_responses=None,
                 admission_control=None,
                 heartbeat_datastores=None,
                 advanced_options=None,
                ):
        """
        :type  enabled: :class:`bool`
        :param enabled: HA Enabled or Disabled Flag to indicate whether or not vSphere HA
            feature is enabled.
        :type  failures_and_responses: :class:`FailuresAndResponses` or ``None``
        :param failures_and_responses: Configuration settings for HA Failures and responses.
            If None or empty, the value is skipped.
        :type  admission_control: :class:`AdmissionControl` or ``None``
        :param admission_control: Configuration settings for HA admission control.
            If None or empty, the value is skipped.
        :type  heartbeat_datastores: :class:`HeartBeatDataStores` or ``None``
        :param heartbeat_datastores: Configuration settings for heart beat data store policy.
            If None or empty, the value is skipped.
        :type  advanced_options: :class:`list` of :class:`com.vmware.appliance.vcenter.settings.v1.config.components.inventory.common_client.OptionValue` or ``None``
        :param advanced_options: Advanced settings.
            If None or empty, the value is skipped.
        """
        self.enabled = enabled
        self.failures_and_responses = failures_and_responses
        self.admission_control = admission_control
        self.heartbeat_datastores = heartbeat_datastores
        self.advanced_options = advanced_options
        VapiStruct.__init__(self)


VsphereHA._set_binding_type(type.StructType(
    'com.vmware.appliance.vcenter.settings.v1.config.components.inventory.cluster.ha.vsphere_HA', {
        'enabled': type.BooleanType(),
        'failures_and_responses': type.OptionalType(type.ReferenceType(__name__, 'FailuresAndResponses')),
        'admission_control': type.OptionalType(type.ReferenceType(__name__, 'AdmissionControl')),
        'heartbeat_datastores': type.OptionalType(type.ReferenceType(__name__, 'HeartBeatDataStores')),
        'advanced_options': type.OptionalType(type.ListType(type.ReferenceType('com.vmware.appliance.vcenter.settings.v1.config.components.inventory.common_client', 'OptionValue'))),
    },
    VsphereHA,
    False,
    None))




class StubFactory(StubFactoryBase):
    _attrs = {
    }




# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Copyright 2023 VMware, Inc.  All rights reserved.

# AUTO GENERATED FILE -- DO NOT MODIFY!
#
# vAPI stub file for package com.vmware.vcenter.namespace_management.cns.
#---------------------------------------------------------------------------

"""
The ``com.vmware.vcenter.namespace_management.cns_client`` module provides
classes for configuration of Persistent Services capabilities on VC clusters.

"""

__author__ = 'VMware, Inc.'
__docformat__ = 'restructuredtext en'

import sys
from warnings import warn

from vmware.vapi.bindings import type
from vmware.vapi.bindings.converter import TypeConverter
from vmware.vapi.bindings.enum import Enum
from vmware.vapi.bindings.error import VapiError
from vmware.vapi.bindings.struct import VapiStruct
from vmware.vapi.bindings.stub import (
    ApiInterfaceStub, StubFactoryBase, VapiInterface)
from vmware.vapi.bindings.common import raise_core_exception
from vmware.vapi.data.validator import (UnionValidator, HasFieldsOfValidator)
from vmware.vapi.exception import CoreException
from vmware.vapi.lib.constants import TaskType
from vmware.vapi.lib.rest import OperationRestMetadata

class MaintenanceActionType(Enum):
    """
    The ``MaintenanceActionType`` class contains actions to be taken when an
    entity enters maintenance mode.

    .. note::
        This class represents an enumerated type in the interface language
        definition. The class contains class attributes which represent the
        values in the current version of the enumerated type. Newer versions of
        the enumerated type may contain new values. To use new values of the
        enumerated type in communication with a server that supports the newer
        version of the API, you instantiate this class. See :ref:`enumerated
        type description page <enumeration_description>`.
    """
    PermanentRemoval = None
    """
    The entity is getting permanently removed. Move applications, rebuild
    storage on other entities before allowing to proceed.

    """
    EnsureAccessibility = None
    """
    The entity is going down temporarily for maintenance. Still need to ensure
    application availability and storage accessibility at least in a degraded
    level.

    """
    NoAction = None
    """
    Admin override to not delay or stop the entity from entering maintenance
    mode.

    """

    def __init__(self, string):
        """
        :type  string: :class:`str`
        :param string: String value for the :class:`MaintenanceActionType` instance.
        """
        Enum.__init__(string)

MaintenanceActionType._set_values({
    'PermanentRemoval': MaintenanceActionType('PermanentRemoval'),
    'EnsureAccessibility': MaintenanceActionType('EnsureAccessibility'),
    'NoAction': MaintenanceActionType('NoAction'),
})
MaintenanceActionType._set_binding_type(type.EnumType(
    'com.vmware.vcenter.namespace_management.cns.maintenance_action_type',
    MaintenanceActionType))





class StubFactory(StubFactoryBase):
    _attrs = {
    }




# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Copyright 2023 VMware, Inc.  All rights reserved.

# AUTO GENERATED FILE -- DO NOT MODIFY!
#
# vAPI stub file for package com.vmware.vcenter.namespace_management.software.
#---------------------------------------------------------------------------

"""
The ``com.vmware.vcenter.namespace_management.software_client`` module provides
classes for managing namespaces software components.

"""

__author__ = 'VMware, Inc.'
__docformat__ = 'restructuredtext en'

import sys
from warnings import warn

from vmware.vapi.bindings import type
from vmware.vapi.bindings.converter import TypeConverter
from vmware.vapi.bindings.enum import Enum
from vmware.vapi.bindings.error import VapiError
from vmware.vapi.bindings.struct import VapiStruct
from vmware.vapi.bindings.stub import (
    ApiInterfaceStub, StubFactoryBase, VapiInterface)
from vmware.vapi.bindings.common import raise_core_exception
from vmware.vapi.data.validator import (UnionValidator, HasFieldsOfValidator)
from vmware.vapi.exception import CoreException
from vmware.vapi.lib.constants import TaskType
from vmware.vapi.lib.rest import OperationRestMetadata


class Clusters(VapiInterface):
    """
    The ``Clusters`` class provides methods to upgrade the vSphere clusters.
    """

    _VAPI_SERVICE_ID = 'com.vmware.vcenter.namespace_management.software.clusters'
    """
    Identifier of the service in canonical form.
    """
    def __init__(self, config):
        """
        :type  config: :class:`vmware.vapi.bindings.stub.StubConfiguration`
        :param config: Configuration to be used for creating the stub.
        """
        VapiInterface.__init__(self, config, _ClustersStub)
        self._VAPI_OPERATION_IDS = {}

    class State(Enum):
        """
        The ``Clusters.State`` class describes the state of the upgrade.

        .. note::
            This class represents an enumerated type in the interface language
            definition. The class contains class attributes which represent the
            values in the current version of the enumerated type. Newer versions of
            the enumerated type may contain new values. To use new values of the
            enumerated type in communication with a server that supports the newer
            version of the API, you instantiate this class. See :ref:`enumerated
            type description page <enumeration_description>`.
        """
        PENDING = None
        """
        Upgrade is in progress.

        """
        READY = None
        """
        Cluster is ready when there is no upgrade or upgrade is completed.

        """
        ERROR = None
        """
        Upgrade failed and need user intervention.

        """

        def __init__(self, string):
            """
            :type  string: :class:`str`
            :param string: String value for the :class:`State` instance.
            """
            Enum.__init__(string)

    State._set_values({
        'PENDING': State('PENDING'),
        'READY': State('READY'),
        'ERROR': State('ERROR'),
    })
    State._set_binding_type(type.EnumType(
        'com.vmware.vcenter.namespace_management.software.clusters.state',
        State))


    class Result(VapiStruct):
        """
        The ``Clusters.Result`` class contains the result of batch upgrade method.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """

        _validator_list = [
            UnionValidator(
                'res',
                {
                    'REJECTED' : [('exception', True)],
                    'STARTED' : [],
                }
            ),
        ]



        def __init__(self,
                     res=None,
                     exception=None,
                    ):
            """
            :type  res: :class:`Clusters.Result.Res`
            :param res: The result of batch upgrade method.
            :type  exception: :class:`Exception`
            :param exception: Exception when cluster pre-check failed during upgrade invocation.
                This attribute is optional and it is only relevant when the value
                of ``res`` is :attr:`Clusters.Result.Res.REJECTED`.
            """
            self.res = res
            self.exception = exception
            VapiStruct.__init__(self)


        class Res(Enum):
            """
            The ``Clusters.Result.Res`` class represents the upgrade invocation result
            for each cluster.

            .. note::
                This class represents an enumerated type in the interface language
                definition. The class contains class attributes which represent the
                values in the current version of the enumerated type. Newer versions of
                the enumerated type may contain new values. To use new values of the
                enumerated type in communication with a server that supports the newer
                version of the API, you instantiate this class. See :ref:`enumerated
                type description page <enumeration_description>`.
            """
            STARTED = None
            """
            Upgrade is started.

            """
            REJECTED = None
            """
            Upgrade is rejected. This implies pre-check failed when invoking upgrade of
            the cluster.

            """

            def __init__(self, string):
                """
                :type  string: :class:`str`
                :param string: String value for the :class:`Res` instance.
                """
                Enum.__init__(string)

        Res._set_values({
            'STARTED': Res('STARTED'),
            'REJECTED': Res('REJECTED'),
        })
        Res._set_binding_type(type.EnumType(
            'com.vmware.vcenter.namespace_management.software.clusters.result.res',
            Res))

    Result._set_binding_type(type.StructType(
        'com.vmware.vcenter.namespace_management.software.clusters.result', {
            'res': type.ReferenceType(__name__, 'Clusters.Result.Res'),
            'exception': type.OptionalType(type.AnyErrorType()),
        },
        Result,
        False,
        None))


    class UpgradeSpec(VapiStruct):
        """
        The ``Clusters.UpgradeSpec`` class contains the specification required to
        upgrade a cluster.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     desired_version=None,
                     ignore_precheck_warnings=None,
                    ):
            """
            :type  desired_version: :class:`str`
            :param desired_version: Version number the cluster is going to be upgraded to.
            :type  ignore_precheck_warnings: :class:`bool` or ``None``
            :param ignore_precheck_warnings: If true, the upgrade workflow will ignore any pre-check warnings
                and proceed with the upgrade.
                If None, the upgrade workflow will not ignore pre-check warnings
                and fail the upgrade. It is equivalent to setting the value to
                false. The workflow adopts a conservative approach of failing the
                upgrade if None to solely let the user decide whether to force the
                upgrade despite the warnings.
            """
            self.desired_version = desired_version
            self.ignore_precheck_warnings = ignore_precheck_warnings
            VapiStruct.__init__(self)


    UpgradeSpec._set_binding_type(type.StructType(
        'com.vmware.vcenter.namespace_management.software.clusters.upgrade_spec', {
            'desired_version': type.StringType(),
            'ignore_precheck_warnings': type.OptionalType(type.BooleanType()),
        },
        UpgradeSpec,
        False,
        None))


    class Summary(VapiStruct):
        """
        The ``Clusters.Summary`` class contains basic information about the cluster
        upgrade related information.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     cluster=None,
                     cluster_name=None,
                     current_version=None,
                     available_versions=None,
                     last_upgraded_date=None,
                     desired_version=None,
                     state=None,
                    ):
            """
            :type  cluster: :class:`str`
            :param cluster: Identifier for the cluster.
                When clients pass a value of this class as a parameter, the
                attribute must be an identifier for the resource type:
                ``ClusterComputeResource``. When methods return a value of this
                class as a return value, the attribute will be an identifier for
                the resource type: ``ClusterComputeResource``.
            :type  cluster_name: :class:`str`
            :param cluster_name: Name of the cluster.
                When clients pass a value of this class as a parameter, the
                attribute must be an identifier for the resource type:
                ``ClusterComputeResource.name``. When methods return a value of
                this class as a return value, the attribute will be an identifier
                for the resource type: ``ClusterComputeResource.name``.
            :type  current_version: :class:`str`
            :param current_version: Current version of the cluster.
            :type  available_versions: :class:`list` of :class:`str`
            :param available_versions: Set of versions available for upgrade.
            :type  last_upgraded_date: :class:`datetime.datetime` or ``None``
            :param last_upgraded_date: Date of last successful upgrade.
                If None, the cluster has not yet been upgraded.
            :type  desired_version: :class:`str` or ``None``
            :param desired_version: Desired version the cluster will be upgraded to.
                If None, the cluster upgrade is not in progress.
            :type  state: :class:`Clusters.State`
            :param state: Current state of the upgrade.
            """
            self.cluster = cluster
            self.cluster_name = cluster_name
            self.current_version = current_version
            self.available_versions = available_versions
            self.last_upgraded_date = last_upgraded_date
            self.desired_version = desired_version
            self.state = state
            VapiStruct.__init__(self)


    Summary._set_binding_type(type.StructType(
        'com.vmware.vcenter.namespace_management.software.clusters.summary', {
            'cluster': type.IdType(resource_types='ClusterComputeResource'),
            'cluster_name': type.IdType(resource_types='ClusterComputeResource.name'),
            'current_version': type.StringType(),
            'available_versions': type.ListType(type.StringType()),
            'last_upgraded_date': type.OptionalType(type.DateTimeType()),
            'desired_version': type.OptionalType(type.StringType()),
            'state': type.ReferenceType(__name__, 'Clusters.State'),
        },
        Summary,
        False,
        None))


    class Info(VapiStruct):
        """
        The ``Clusters.Info`` class contains detailed information about the cluster
        upgrade status and related information.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     current_version=None,
                     available_versions=None,
                     last_upgraded_date=None,
                     messages=None,
                     state=None,
                     upgrade_status=None,
                    ):
            """
            :type  current_version: :class:`str`
            :param current_version: Current version of the cluster.
            :type  available_versions: :class:`list` of :class:`str`
            :param available_versions: Set of available versions can be upgraded to.
            :type  last_upgraded_date: :class:`datetime.datetime` or ``None``
            :param last_upgraded_date: Date of last successful upgrade.
                If None, the cluster has not yet been upgraded.
            :type  messages: :class:`list` of :class:`Clusters.Message`
            :param messages: Current set of messages associated with the cluster version.
            :type  state: :class:`Clusters.State`
            :param state: Current state of the upgrade.
            :type  upgrade_status: :class:`Clusters.UpgradeStatus` or ``None``
            :param upgrade_status: Information about upgrade in progress.
                If None, the cluster upgrade is not in progress.
            """
            self.current_version = current_version
            self.available_versions = available_versions
            self.last_upgraded_date = last_upgraded_date
            self.messages = messages
            self.state = state
            self.upgrade_status = upgrade_status
            VapiStruct.__init__(self)


    Info._set_binding_type(type.StructType(
        'com.vmware.vcenter.namespace_management.software.clusters.info', {
            'current_version': type.StringType(),
            'available_versions': type.ListType(type.StringType()),
            'last_upgraded_date': type.OptionalType(type.DateTimeType()),
            'messages': type.ListType(type.ReferenceType(__name__, 'Clusters.Message')),
            'state': type.ReferenceType(__name__, 'Clusters.State'),
            'upgrade_status': type.OptionalType(type.ReferenceType(__name__, 'Clusters.UpgradeStatus')),
        },
        Info,
        False,
        None))


    class UpgradeStatus(VapiStruct):
        """
        The ``Clusters.UpgradeStatus`` class contains detailed information about
        the cluster when upgraded is in progress.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     desired_version=None,
                     messages=None,
                     progress=None,
                    ):
            """
            :type  desired_version: :class:`str` or ``None``
            :param desired_version: Desired version the cluster will be upgraded to.
                If None, the cluster upgrade is not in progress.
            :type  messages: :class:`list` of :class:`Clusters.Message`
            :param messages: Current set of messages associated with the upgrade state.
            :type  progress: :class:`Clusters.UpgradeProgress` or ``None``
            :param progress: Information about upgrade progess.
                If None, the cluster upgrade is not in progress.
            """
            self.desired_version = desired_version
            self.messages = messages
            self.progress = progress
            VapiStruct.__init__(self)


    UpgradeStatus._set_binding_type(type.StructType(
        'com.vmware.vcenter.namespace_management.software.clusters.upgrade_status', {
            'desired_version': type.OptionalType(type.StringType()),
            'messages': type.ListType(type.ReferenceType(__name__, 'Clusters.Message')),
            'progress': type.OptionalType(type.ReferenceType(__name__, 'Clusters.UpgradeProgress')),
        },
        UpgradeStatus,
        False,
        None))


    class UpgradeProgress(VapiStruct):
        """
        The ``Clusters.UpgradeProgress`` class contains detailed information about
        the cluster upgrade progess.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     total=None,
                     completed=None,
                     message=None,
                    ):
            """
            :type  total: :class:`long`
            :param total: Total amount of the work for the operation. The work here
                represents the number of master nodes in the cluster need to be
                upgraded.
            :type  completed: :class:`long`
            :param completed: The amount of work completed for the operation. The value can only
                be incremented. The number or master nodes which upgrade completed.
            :type  message: :class:`com.vmware.vapi.std_client.LocalizableMessage`
            :param message: Message about the work progress.
            """
            self.total = total
            self.completed = completed
            self.message = message
            VapiStruct.__init__(self)


    UpgradeProgress._set_binding_type(type.StructType(
        'com.vmware.vcenter.namespace_management.software.clusters.upgrade_progress', {
            'total': type.IntegerType(),
            'completed': type.IntegerType(),
            'message': type.ReferenceType('com.vmware.vapi.std_client', 'LocalizableMessage'),
        },
        UpgradeProgress,
        False,
        None))


    class Message(VapiStruct):
        """
        The ``Clusters.Message`` class contains the information about the object
        configuration.

        .. tip::
            The arguments are used to initialize data attributes with the same
            names.
        """




        def __init__(self,
                     severity=None,
                     details=None,
                    ):
            """
            :type  severity: :class:`Clusters.Message.Severity`
            :param severity: Type of the message.
            :type  details: :class:`com.vmware.vapi.std_client.LocalizableMessage`
            :param details: Details about the message.
            """
            self.severity = severity
            self.details = details
            VapiStruct.__init__(self)


        class Severity(Enum):
            """
            The ``Clusters.Message.Severity`` class represents the severity of the
            message.

            .. note::
                This class represents an enumerated type in the interface language
                definition. The class contains class attributes which represent the
                values in the current version of the enumerated type. Newer versions of
                the enumerated type may contain new values. To use new values of the
                enumerated type in communication with a server that supports the newer
                version of the API, you instantiate this class. See :ref:`enumerated
                type description page <enumeration_description>`.
            """
            INFO = None
            """
            Informational message. This may be accompanied by vCenter event.

            """
            WARNING = None
            """
            Warning message. This may be accompanied by vCenter event.

            """
            ERROR = None
            """
            Error message. This is accompanied by vCenter event and/or alarm.

            """

            def __init__(self, string):
                """
                :type  string: :class:`str`
                :param string: String value for the :class:`Severity` instance.
                """
                Enum.__init__(string)

        Severity._set_values({
            'INFO': Severity('INFO'),
            'WARNING': Severity('WARNING'),
            'ERROR': Severity('ERROR'),
        })
        Severity._set_binding_type(type.EnumType(
            'com.vmware.vcenter.namespace_management.software.clusters.message.severity',
            Severity))

    Message._set_binding_type(type.StructType(
        'com.vmware.vcenter.namespace_management.software.clusters.message', {
            'severity': type.ReferenceType(__name__, 'Clusters.Message.Severity'),
            'details': type.ReferenceType('com.vmware.vapi.std_client', 'LocalizableMessage'),
        },
        Message,
        False,
        None))



    def upgrade(self,
                cluster,
                spec,
                ):
        """
        Upgrade the cluster to a specific version. This operation upgrades the
        components on control plane VMs and worker plane hosts based on the
        selected version. Before upgrading, this operation performs pre-checks
        and sets the evaluation response in Info.UpgradeStatus.messages with
        various Message.Severity levels. Depending on the severity, the upgrade
        may or may not proceed beyond prechecks. Here is a list of severities
        and corresponding behavior: - ERROR: Upgrade does not proceed beyond
        precheck operation - WARNING: Upgrade proceeds beyond precheck
        operation only if UpgradeSpec.ignorePrecheckWarnings is set to true -
        INFO: Upgrade proceeds beyond precheck operation uninterrupted

        :type  cluster: :class:`str`
        :param cluster: Identifier for the cluster which will be upgraded.
            The parameter must be an identifier for the resource type:
            ``ClusterComputeResource``.
        :type  spec: :class:`Clusters.UpgradeSpec`
        :param spec: Specification for upgrading the cluster.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            if the system reports an error while responding to the request.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unsupported` 
            if pre-check failed of the cluster.
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            if cluster could not be located.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            if the user can not be authenticated.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            if the user does not have Namespaces.Upgrade privilege.
        """
        return self._invoke('upgrade',
                            {
                            'cluster': cluster,
                            'spec': spec,
                            })

    def upgrade_multiple(self,
                         specs,
                         ):
        """
        Upgrade a set of clusters to its corresponding specific version.

        :type  specs: :class:`dict` of :class:`str` and :class:`Clusters.UpgradeSpec`
        :param specs: Specifications for upgrading selected clusters.
            The key in the parameter :class:`dict` must be an identifier for
            the resource type: ``ClusterComputeResource``.
        :rtype: :class:`dict` of :class:`str` and :class:`Clusters.Result`
        :return: Pre-check result when invoking upgrade for each cluster.
            The key in the return value :class:`dict` will be an identifier for
            the resource type: ``ClusterComputeResource``.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            if the system reports an error while responding to the request.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            if the user can not be authenticated.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            if the user does not have Namespaces.Upgrade privilege on all
            specified clusters.
        """
        return self._invoke('upgrade_multiple',
                            {
                            'specs': specs,
                            })

    def get(self,
            cluster,
            ):
        """
        Returns upgrade related information of a specific cluster.

        :type  cluster: :class:`str`
        :param cluster: Identifier for the cluster which will be upgraded.
            The parameter must be an identifier for the resource type:
            ``ClusterComputeResource``.
        :rtype: :class:`Clusters.Info`
        :return: Information about the upgrade of the specified WCP enabled cluster.
        :raise: :class:`com.vmware.vapi.std.errors_client.NotFound` 
            if cluster could not be located.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            if the system reports an error while responding to the request.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            if the user can not be authenticated.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            if the user does not have System.Read privilege.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unsupported` 
            if the cluster is not WCP enabled.
        """
        return self._invoke('get',
                            {
                            'cluster': cluster,
                            })

    def list(self):
        """
        Returns upgrade related information about all WCP enabled clusters.


        :rtype: :class:`list` of :class:`Clusters.Summary`
        :return: List of upgrade summary of all WCP enabled clusters.
        :raise: :class:`com.vmware.vapi.std.errors_client.Error` 
            if the system reports an error while responding to the request.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthenticated` 
            if the user can not be authenticated.
        :raise: :class:`com.vmware.vapi.std.errors_client.Unauthorized` 
            if the user does not have System.Read privilege.
        """
        return self._invoke('list', None)
class _ClustersStub(ApiInterfaceStub):
    def __init__(self, config):
        # properties for upgrade operation
        upgrade_input_type = type.StructType('operation-input', {
            'cluster': type.IdType(resource_types='ClusterComputeResource'),
            'spec': type.ReferenceType(__name__, 'Clusters.UpgradeSpec'),
        })
        upgrade_error_dict = {
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),
            'com.vmware.vapi.std.errors.unsupported':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unsupported'),
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),

        }
        upgrade_input_value_validator_list = [
        ]
        upgrade_output_validator_list = [
        ]
        upgrade_rest_metadata = OperationRestMetadata(
            http_method='POST',
            url_template='/vcenter/namespace-management/software/clusters/{cluster}',
            request_body_parameter='spec',
            path_variables={
                'cluster': 'cluster',
            },
            query_parameters={
            },
            dispatch_parameters={
                'action': 'upgrade',
            },
            header_parameters={
            },
            dispatch_header_parameters={
            }
        )

        # properties for upgrade_multiple operation
        upgrade_multiple_input_type = type.StructType('operation-input', {
            'specs': type.MapType(type.IdType(), type.ReferenceType(__name__, 'Clusters.UpgradeSpec')),
        })
        upgrade_multiple_error_dict = {
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),

        }
        upgrade_multiple_input_value_validator_list = [
        ]
        upgrade_multiple_output_validator_list = [
        ]
        upgrade_multiple_rest_metadata = OperationRestMetadata(
            http_method='POST',
            url_template='/vcenter/namespace-management/software/clusters',
            request_body_parameter='specs',
            path_variables={
            },
            query_parameters={
            },
            dispatch_parameters={
                'action': 'upgradeMultiple',
            },
            header_parameters={
            },
            dispatch_header_parameters={
            }
        )

        # properties for get operation
        get_input_type = type.StructType('operation-input', {
            'cluster': type.IdType(resource_types='ClusterComputeResource'),
        })
        get_error_dict = {
            'com.vmware.vapi.std.errors.not_found':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'NotFound'),
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),
            'com.vmware.vapi.std.errors.unsupported':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unsupported'),

        }
        get_input_value_validator_list = [
        ]
        get_output_validator_list = [
        ]
        get_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/vcenter/namespace-management/software/clusters/{cluster}',
            path_variables={
                'cluster': 'cluster',
            },
            query_parameters={
            },
            dispatch_parameters={
            },
            header_parameters={
            },
            dispatch_header_parameters={
            }
        )

        # properties for list operation
        list_input_type = type.StructType('operation-input', {})
        list_error_dict = {
            'com.vmware.vapi.std.errors.error':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Error'),
            'com.vmware.vapi.std.errors.unauthenticated':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthenticated'),
            'com.vmware.vapi.std.errors.unauthorized':
                type.ReferenceType('com.vmware.vapi.std.errors_client', 'Unauthorized'),

        }
        list_input_value_validator_list = [
        ]
        list_output_validator_list = [
        ]
        list_rest_metadata = OperationRestMetadata(
            http_method='GET',
            url_template='/vcenter/namespace-management/software/clusters',
            path_variables={
            },
            query_parameters={
            },
            dispatch_parameters={
            },
            header_parameters={
            },
            dispatch_header_parameters={
            }
        )

        operations = {
            'upgrade': {
                'input_type': upgrade_input_type,
                'output_type': type.VoidType(),
                'errors': upgrade_error_dict,
                'input_value_validator_list': upgrade_input_value_validator_list,
                'output_validator_list': upgrade_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'upgrade_multiple': {
                'input_type': upgrade_multiple_input_type,
                'output_type': type.MapType(type.IdType(), type.ReferenceType(__name__, 'Clusters.Result')),
                'errors': upgrade_multiple_error_dict,
                'input_value_validator_list': upgrade_multiple_input_value_validator_list,
                'output_validator_list': upgrade_multiple_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'get': {
                'input_type': get_input_type,
                'output_type': type.ReferenceType(__name__, 'Clusters.Info'),
                'errors': get_error_dict,
                'input_value_validator_list': get_input_value_validator_list,
                'output_validator_list': get_output_validator_list,
                'task_type': TaskType.NONE,
            },
            'list': {
                'input_type': list_input_type,
                'output_type': type.ListType(type.ReferenceType(__name__, 'Clusters.Summary')),
                'errors': list_error_dict,
                'input_value_validator_list': list_input_value_validator_list,
                'output_validator_list': list_output_validator_list,
                'task_type': TaskType.NONE,
            },
        }
        rest_metadata = {
            'upgrade': upgrade_rest_metadata,
            'upgrade_multiple': upgrade_multiple_rest_metadata,
            'get': get_rest_metadata,
            'list': list_rest_metadata,
        }
        ApiInterfaceStub.__init__(
            self, iface_name='com.vmware.vcenter.namespace_management.software.clusters',
            config=config, operations=operations, rest_metadata=rest_metadata,
            is_vapi_rest=True)


class StubFactory(StubFactoryBase):
    _attrs = {
        'Clusters': Clusters,
    }

